# -*- coding: utf-8 -*-

from .type import SignType, WeChatPayType


class WeChatPay():
    def __init__(self,
                 wechatpay_type,
                 mchid,
                 private_key,
                 cert_serial_no,
                 appid,
                 apiv3_key,
                 notify_url=None,
                 cert_dir=None,
                 logger=None,
                 partner_mode=False,
                 proxy=None):
        """
        :param wechatpay_type: 微信支付类型，示例值:WeChatPayType.MINIPROG
        :param mchid: 直连商户号，示例值:'1230000109'
        :param private_key: 商户证书私钥，示例值:'MIIEvwIBADANBgkqhkiG9w0BAQE...'
        :param cert_serial_no: 商户证书序列号，示例值:'444F4864EA9B34415...'
        :param appid: 应用ID，示例值:'wxd678efh567hg6787'
        :param apiv3_key: 商户APIv3密钥，示例值:'a12d3924fd499edac8a5efc...'
        :param notify_url: 通知地址，示例值:'https://www.weixin.qq.com/wxpay/pay.php'
        :param cert_dir: 平台证书存放目录，示例值:'/server/cert'
        :param logger: 日志记录器，示例值logging.getLoger('demo')
        :param partner_mode: 接入模式，默认False为直连商户模式，True为服务商模式
        :param proxy: 代理设置，示例值:{"https": "http://10.10.1.10:1080"}
        """
        from .core import Core

        self._type = wechatpay_type
        self._mchid = mchid
        self._appid = appid
        self._notify_url = notify_url
        self._core = Core(mchid=self._mchid,
                          cert_serial_no=cert_serial_no,
                          private_key=private_key,
                          apiv3_key=apiv3_key,
                          cert_dir=cert_dir,
                          logger=logger,
                          proxy=proxy)
        self._partner_mode = partner_mode

    def sign(self, data, sign_type=SignType.RSA_SHA256):
        """使用RSAwithSHA256或HMAC_256算法计算签名值供调起支付时使用
        :param data: 需要签名的参数清单
        :微信支付订单采用RSAwithSHA256算法时，示例值:['wx888','1414561699','5K8264ILTKCH16CQ2502S....','prepay_id=wx201410272009395522657....']
        :微信支付分订单采用HMAC_SHA256算法时，示例值:{'mch_id':'1230000109','service_id':'88888888000011','out_order_no':'1234323JKHDFE1243252'}
        """
        return self._core.sign(data, sign_type)

    def decrypt_callback(self, headers, body):
        """解密回调接口收到的信息，仅返回resource解密后的参数字符串，此接口为兼容旧版本而保留，建议调用callback()
        :param headers: 回调接口收到的headers
        :param body: 回调接口收到的body
        """
        return self._core.decrypt_callback(headers, body)

    def callback(self, headers, body):
        """解密回调接口收到的信息，返回所有传入的参数
        :param headers: 回调接口收到的headers
        :param body: 回调接口收到的body
        """
        return self._core.callback(headers, body)

    def decrypt(self, ciphtext):
        """解密微信支付平台返回的信息中的敏感字段
        :param ciphtext: 加密后的敏感字段，示例值:'Qe41VhP/sGdNeTHMQGlxCWiUyHu6XNO9GCYln2Luv4HhwJzZBfcL12sB+PgZcS5NhePBog30NgJ1xRaK+gbGDKwpg=='
        """
        return self._core.decrypt(ciphtext)

    from .apply4subject import (apply4subject_cancel, apply4subject_query,
                                apply4subject_state, apply4subject_submit)
    from .applyment import (applyment_query, applyment_settlement_modify,
                            applyment_settlement_query, applyment_submit)
    from .businesscircle import (business_parking_sync, business_point_status,
                                 points_notify, user_authorization)
    from .capital import (capital_branches, capital_cities,
                          capital_corporate_banks, capital_personal_banks,
                          capital_provinces, capital_search_bank_number)
    from .complaint import (complaint_complete, complaint_detail_query,
                            complaint_history_query, complaint_image_download,
                            complaint_image_upload, complaint_list_query,
                            complaint_notification_create,
                            complaint_notification_delete,
                            complaint_notification_query,
                            complaint_notification_update, complaint_response,
                            complaint_update_refund)
    from .fapiao import (fapiao_applications, fapiao_card_template,
                         fapiao_insert_cards, fapiao_merchant_base_info,
                         fapiao_merchant_config, fapiao_query, fapiao_reverse,
                         fapiao_set_merchant_config, fapiao_tax_codes,
                         fapiao_title, fapiao_title_url, fapiao_upload_file)
    from .goldplan import (goldplan_advertising_close,
                           goldplan_advertising_filter,
                           goldplan_advertising_open,
                           goldplan_custompage_change, goldplan_plan_change)
    from .marketing import (marketing_busifavor_callback_query,
                            marketing_busifavor_callback_update,
                            marketing_busifavor_coupon_associate,
                            marketing_busifavor_coupon_deactivate,
                            marketing_busifavor_coupon_detail,
                            marketing_busifavor_coupon_disassociate,
                            marketing_busifavor_coupon_return,
                            marketing_busifavor_coupon_use,
                            marketing_busifavor_couponcode_upload,
                            marketing_busifavor_stock_budget,
                            marketing_busifavor_stock_create,
                            marketing_busifavor_stock_modify,
                            marketing_busifavor_stock_query,
                            marketing_busifavor_subsidy_pay,
                            marketing_busifavor_subsidy_query,
                            marketing_busifavor_user_coupon,
                            marketing_card_send,
                            marketing_favor_callback_update,
                            marketing_favor_coupon_detail,
                            marketing_favor_refund_flow,
                            marketing_favor_stock_create,
                            marketing_favor_stock_detail,
                            marketing_favor_stock_item,
                            marketing_favor_stock_list,
                            marketing_favor_stock_merchant,
                            marketing_favor_stock_pause,
                            marketing_favor_stock_restart,
                            marketing_favor_stock_send,
                            marketing_favor_stock_start,
                            marketing_favor_use_flow,
                            marketing_favor_user_coupon,
                            marketing_image_upload,
                            marketing_partnership_build,
                            marketing_partnership_query,
                            marketing_paygift_activity_create,
                            marketing_paygift_activity_detail,
                            marketing_paygift_activity_list,
                            marketing_paygift_activity_terminate,
                            marketing_paygift_goods_list,
                            marketing_paygift_merchant_add,
                            marketing_paygift_merchant_delete,
                            marketing_paygift_merchants_list)
    from .media import image_upload, video_upload
    from .merchantrisk import (merchantrisk_callback_create,
                               merchantrisk_callback_delete,
                               merchantrisk_callback_query,
                               merchantrisk_callback_update)
    from .parking import (parking_enter, parking_order, parking_order_query,
                          parking_service_find)
    from .payscore import (payscore_cancel, payscore_complete, payscore_create,
                           payscore_direct_complete, payscore_merchant_bill,
                           payscore_modify, payscore_pay, payscore_permission,
                           payscore_permission_query,
                           payscore_permission_terminate, payscore_query,
                           payscore_refund, payscore_refund_query,
                           payscore_sync)
    from .profitsharing import (brand_profitsharing_add_receiver,
                                brand_profitsharing_amount_query,
                                brand_profitsharing_config_query,
                                brand_profitsharing_delete_receiver,
                                brand_profitsharing_order,
                                brand_profitsharing_order_query,
                                brand_profitsharing_return,
                                brand_profitsharing_return_query,
                                brand_profitsharing_unfreeze,
                                profitsharing_add_receiver,
                                profitsharing_amount_query, profitsharing_bill,
                                profitsharing_config_query,
                                profitsharing_delete_receiver,
                                profitsharing_order, profitsharing_order_query,
                                profitsharing_return,
                                profitsharing_return_query,
                                profitsharing_unfreeze)
    from .smartguide import (guides_assign, guides_query, guides_register,
                             guides_update)
    from .transaction import (close, combine_close, combine_pay, combine_query,
                              download_bill, fundflow_bill, pay, query,
                              query_refund, refund, submch_fundflow_bill,
                              trade_bill)
    from .transfer import (transfer_batch, transfer_bill_receipt,
                           transfer_detail_receipt, transfer_query_batchid,
                           transfer_query_bill_receipt,
                           transfer_query_detail_id,
                           transfer_query_out_batch_no,
                           transfer_query_out_detail_no,
                           transfer_query_receipt)



# -*- coding: utf-8 -*-

from .type import RequestType


def applyment_submit(self, business_code, contact_info, subject_info, business_info, settlement_info, bank_account_info, addition_info=None):
    """提交申请单
    :param business_code: 业务申请编号，示例值:'APPLYMENT_00000000001'
    :param contact_info: 超级管理员信息，示例值:{'contact_name':'张三','contact_id_number':'320311770706001','mobile_phone':'13900000000','contact_email':'admin@demo.com'}
    :param subject_info: 主体资料，示例值:{'subject_type':'SUBJECT_TYPE_ENTERPRISE','business_license_info':{'license_copy':'demo-media-id','license_number':'123456789012345678','merchant_name':'腾讯科技有限公司','legal_person':'张三'},'identity_info':{'id_doc_type':'IDENTIFICATION_TYPE_IDCARD','id_card_info'{'id_card_copy':'demo-media-id'}}}
    :param business_info: 经营资料，示例值:{'merchant_shortname':'张三餐饮店','service_phone':'0758xxxxxx','sales_info':{'sales_scenes_type':['SALES_SCENES_STORE','SALES_SCENES_MP']}}
    :param settlement_info: 结算规则，示例值:{'settlement_id':'719','qualification_type':'餐饮'}
    :param bank_account_info: 结算银行账户，示例值:{'bank_account_type':'BANK_ACCOUNT_TYPE_CORPORATE','account_name':'xx公司','account_bank':'工商银行','bank_address_code':'110000','account_number':'1234567890'}
    :param addition_info: 补充材料，示例值:{'legal_person_commitment':'demo-media-id'}
    """
    params = {}
    if business_code:
        params.update({'business_code': business_code})
    else:
        raise Exception('business_code is not assigned.')
    if contact_info:
        params.update({'contact_info': contact_info})
    else:
        raise Exception('contact_info is not assigned.')
    if subject_info:
        params.update({'subject_info': subject_info})
    else:
        raise Exception('subject_info is not assigned.')
    if business_info:
        params.update({'business_info': business_info})
    else:
        raise Exception('business_info is not assigned')
    if settlement_info:
        params.update({'settlement_info': settlement_info})
    else:
        raise Exception('settlement_info is not assigned.')
    if bank_account_info:
        params.update({'bank_account_info': bank_account_info})
    else:
        raise Exception('bank_account_info is not assigned.')
    if addition_info:
        params.update({'addition_info': addition_info})
    if params.get('contact_info').get('contact_name'):
        params['contact_info']['contact_name'] = self._core.encrypt(params['contact_info']['contact_name'])
    if params.get('contact_info').get('contact_id_number'):
        params['contact_info']['contact_id_number'] = self._core.encrypt(params['contact_info']['contact_id_number'])
    if params.get('contact_info').get('openid'):
        params['contact_info']['openid'] = self._core.encrypt(params['contact_info']['openid'])
    if params.get('contact_info').get('mobile_phone'):
        params['contact_info']['mobile_phone'] = self._core.encrypt(params['contact_info']['mobile_phone'])
    if params.get('contact_info').get('contact_email'):
        params['contact_info']['contact_email'] = self._core.encrypt(params['contact_info']['contact_email'])
    id_card_name = params.get('subject_info').get('identity_info').get('id_card_info', {}).get('id_card_name')
    if id_card_name:
        params['subject_info']['identity_info']['id_card_info']['id_card_name'] = self._core.encrypt(id_card_name)
    id_card_number = params.get('subject_info').get('identity_info').get('id_card_info', {}).get('id_card_number')
    if id_card_number:
        params['subject_info']['identity_info']['id_card_info']['id_card_number'] = self._core.encrypt(id_card_number)
    id_card_address = params.get('subject_info').get('identity_info').get('id_card_info', {}).get('id_card_address')
    if id_card_address:
        params['subject_info']['identity_info']['id_card_info']['id_card_address'] = self._core.encrypt(id_card_address)
    id_doc_name = params.get('subject_info').get('identity_info').get('id_doc_info', {}).get('id_doc_name')
    if id_doc_name:
        params['subject_info']['identity_info']['id_doc_info']['id_doc_name'] = self._core.encrypt(id_doc_name)
    id_doc_number = params.get('subject_info').get('identity_info').get('id_doc_info', {}).get('id_doc_number')
    if id_doc_number:
        params['subject_info']['identity_info']['id_doc_info']['id_doc_number'] = self._core.encrypt(id_doc_number)
    id_doc_address = params.get('subject_info').get('identity_info').get('id_doc_info', {}).get('id_doc_address')
    if id_doc_address:
        params['subject_info']['identity_info']['id_doc_info']['id_doc_address'] = self._core.encrypt(id_doc_address)
    if params.get('subject_info').get('ubo_info_list'):
        for ubo_info in params['subject_info']['ubo_info_list']:
            ubo_info['ubo_id_doc_name'] = self._core.encrypt(ubo_info['ubo_id_doc_name'])
            ubo_info['ubo_id_doc_number'] = self._core.encrypt(ubo_info['ubo_id_doc_number'])
            ubo_info['ubo_id_doc_address'] = self._core.encrypt(ubo_info['ubo_id_doc_address'])
    params['bank_account_info']['account_name'] = self._core.encrypt(params['bank_account_info']['account_name'])
    params['bank_account_info']['account_number'] = self._core.encrypt(params['bank_account_info']['account_number'])
    path = '/v3/applyment4sub/applyment/'
    return self._core.request(path, method=RequestType.POST, data=params, cipher_data=True)


def applyment_query(self, business_code=None, applyment_id=None):
    """查询申请单状态
    :param business_code: 业务申请编号，示例值:'APPLYMENT_00000000001'
    :param applyment_id: 申请单号，示例值:2000001234567890
    """
    if business_code:
        path = '/v3/applyment4sub/applyment/business_code/%s' % business_code
    elif applyment_id:
        path = '/v3/applyment4sub/applyment/applyment_id/%s' % applyment_id
    else:
        raise Exception('business_code or applyment_id is not assigned.')
    return self._core.request(path)


def applyment_settlement_modify(self, sub_mchid, account_type, account_bank, bank_address_code, account_number, bank_name=None, bank_branch_id=None):
    """修改结算账号
    :param sub_mchid: 特约商户号，示例值:'1511101111'
    :param account_type: 账户类型，枚举值:'ACCOUNT_TYPE_BUSINESS':对公银行账户，'ACCOUNT_TYPE_PRIVATE':经营者个人银行卡。示例值:'ACCOUNT_TYPE_BUSINESS'
    :param account_bank: 开户银行，示例值:'工商银行'
    :param bank_address_code: 开户银行省市编码，示例值:'110000'
    :param account_number: 银行账号，示例值:'1234567890'
    :param bank_name: 开户银行全称（含支行），示例值:'施秉县农村信用合作联社城关信用社'
    :param bank_branch_id: 开户银行联行号，示例值:'402713354941'
    """
    params = {}
    if sub_mchid:
        path = '/v3/apply4sub/sub_merchants/%s/modify-settlement' % sub_mchid
    else:
        raise Exception('sub_mchid is not assigned.')
    if account_type:
        params.update({'account_type': account_type})
    else:
        raise Exception('account_type is not assigned.')
    if account_bank:
        params.update({'account_bank': account_bank})
    else:
        raise Exception('account_bank is not assigned.')
    if bank_address_code:
        params.update({'bank_address_code': bank_address_code})
    else:
        raise Exception('bank_address_code is not assigned.')
    cipher_data = False
    if account_number:
        params.update({'account_number': self._core.encrypt(account_number)})
        cipher_data = True
    else:
        raise Exception('account_number is not assigned.')
    if bank_name:
        params.update({'bank_name': bank_name})
    if bank_branch_id:
        params.update({'bank_branch_id': bank_branch_id})
    return self._core.request(path, method=RequestType.POST, data=params, cipher_data=cipher_data)


def applyment_settlement_query(self, sub_mchid):
    """查询结算账户
    :param sub_mchid: 特约商户号，示例值:'1511101111'
    """
    if sub_mchid:
        path = '/v3/apply4sub/sub_merchants/%s/settlement' % sub_mchid
    else:
        raise Exception('sub_mchid is not assigned.')
    return self._core.request(path)



# -*- coding: utf-8 -*-

from .type import RequestType, WeChatPayType


def pay(self,
        description,
        out_trade_no,
        amount,
        payer=None,
        time_expire=None,
        attach=None,
        goods_tag=None,
        detail=None,
        scene_info=None,
        settle_info=None,
        notify_url=None,
        appid=None,
        mchid=None,
        sub_appid=None,
        sub_mchid=None,
        support_fapiao=False,
        pay_type=None):
    """统一下单
    :return code, message:
    :param description: 商品描述，示例值:'Image形象店-深圳腾大-QQ公仔'
    :param out_trade_no: 商户订单号，示例值:'1217752501201407033233368018'
    :param amount: 订单金额，示例值:{'total':100, 'currency':'CNY'}
    :param payer: 支付者，示例值:{'openid':'oHkLxtx0vUqe-18p_AXTZ1innxkCY'}
    :param time_expire: 交易结束时间，示例值:'2018-06-08T10:34:56+08:00'
    :param attach: 附加数据，示例值:'自定义数据'
    :param goods_tag: 订单优惠标记，示例值:'WXG'
    :param detail: 优惠功能，示例值:{'cost_price':608800, 'invoice_id':'微信123', 'goods_detail':[{'merchant_goods_id':'商品编码', 'wechatpay_goods_id':'1001', 'goods_name':'iPhoneX 256G', 'quantity':1, 'unit_price':828800}]}
    :param scene_info: 场景信息，示例值:{'payer_client_ip':'14.23.150.211', 'device_id':'013467007045764', 'store_info':{'id':'0001', 'name':'腾讯大厦分店', 'area_code':'440305', 'address':'广东省深圳市南山区科技中一道10000号'}}
    :param settle_info: 结算信息，示例值:{'profit_sharing':False}
    :param notify_url: 通知地址，示例值:'https://www.weixin.qq.com/wxpay/pay.php'
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    :param mchid: 微信支付商户号，可不填，默认传入初始化的mchid，示例值:'987654321'
    :param sub_appid: (服务商模式)子商户应用ID，示例值:'wxd678efh567hg6999'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    :param support_fapiao: 电子发票入口开放标识，传入true时，支付成功消息和支付详情页将出现开票入口。
    :param pay_type: 微信支付类型，示例值:WeChatPayType.JSAPI
    """
    params = {}
    if not (notify_url or self._notify_url):
        raise Exception('notify_url is not assigned.')
    params.update({'notify_url': notify_url or self._notify_url})
    if description:
        params.update({'description': description})
    else:
        raise Exception('description is not assigned.')
    if out_trade_no:
        params.update({'out_trade_no': out_trade_no})
    else:
        raise Exception('out_trade_no is not assigned.')
    if amount:
        params.update({'amount': amount})
    else:
        raise Exception('amount is not assigned.')
    if payer:
        params.update({'payer': payer})
    if scene_info:
        params.update({'scene_info': scene_info})
    if time_expire:
        params.update({'time_expire': time_expire})
    if attach:
        params.update({'attach': attach})
    if goods_tag:
        params.update({'goods_tag': goods_tag})
    if detail:
        params.update({'detail': detail})
    if settle_info:
        params.update({'settle_info': settle_info})
    pay_type = pay_type or self._type
    if self._partner_mode:
        params.update({'sp_appid': appid or self._appid})
        params.update({'sp_mchid': mchid or self._mchid})
        if sub_mchid:
            params.update({'sub_mchid': sub_mchid})
        else:
            raise Exception('sub_mchid is not assigned.')
        if sub_appid:
            params.update({'sub_appid': sub_appid})
        if pay_type in [WeChatPayType.JSAPI, WeChatPayType.MINIPROG]:
            if not payer:
                raise Exception('payer is not assigned')
            path = '/v3/pay/partner/transactions/jsapi'
        elif pay_type == WeChatPayType.APP:
            path = '/v3/pay/partner/transactions/app'
        elif pay_type == WeChatPayType.H5:
            if not scene_info:
                raise Exception('scene_info is not assigned.')
            path = '/v3/pay/partner/transactions/h5'
        elif pay_type == WeChatPayType.NATIVE:
            path = '/v3/pay/partner/transactions/native'
        else:
            raise Exception('pay_type is not assigned.')
    else:
        params.update({'appid': appid or self._appid})
        params.update({'mchid': mchid or self._mchid})
        if pay_type in [WeChatPayType.JSAPI, WeChatPayType.MINIPROG]:
            if not payer:
                raise Exception('payer is not assigned')
            path = '/v3/pay/transactions/jsapi'
        elif pay_type == WeChatPayType.APP:
            path = '/v3/pay/transactions/app'
        elif pay_type == WeChatPayType.H5:
            if not scene_info:
                raise Exception('scene_info is not assigned.')
            path = '/v3/pay/transactions/h5'
        elif pay_type == WeChatPayType.NATIVE:
            path = '/v3/pay/transactions/native'
        else:
            raise Exception('pay_type is not assigned.')
    if support_fapiao:
        params.update({'support_fapiao': support_fapiao})
    return self._core.request(path, method=RequestType.POST, data=params)


def close(self, out_trade_no, mchid=None, sub_mchid=None):
    """关闭订单
    :param out_trade_no: 商户订单号，示例值:'1217752501201407033233368018'
    :param mchid: 微信支付商户号，可不传，默认传入初始化的mchid。示例值:'987654321'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    if self._partner_mode:
        if out_trade_no:
            path = '/v3/pay/partner/transactions/out-trade-no/%s/close' % out_trade_no
        else:
            raise Exception('out_trade_no is not assigned.')
        if sub_mchid:
            params = {'sp_mchid': mchid or self._mchid, 'sub_mchid': sub_mchid}
        else:
            raise Exception('sub_mchid is not assigned.')
    else:
        if out_trade_no:
            path = '/v3/pay/transactions/out-trade-no/%s/close' % out_trade_no
        else:
            raise Exception('out_trade_no is not assigned.')
        params = {'mchid': mchid or self._mchid}
    return self._core.request(path, method=RequestType.POST, data=params)


def query(self, transaction_id=None, out_trade_no=None, mchid=None, sub_mchid=None):
    """查询订单
    :param transaction_id: 微信支付订单号，示例值:1217752501201407033233368018
    :param out_trade_no: 商户订单号，示例值:1217752501201407033233368018
    :param mchid: 微信支付商户号，可不传，默认传入初始化的mchid。示例值:'987654321'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    if self._partner_mode:
        if transaction_id:
            path = '/v3/pay/partner/transactions/id/%s' % transaction_id
        elif out_trade_no:
            path = '/v3/pay/partner/transactions/out-trade-no/%s' % out_trade_no
        else:
            raise Exception('transaction_id or out_trade_no is not assigned.')
        path = '%s?sp_mchid=%s&sub_mchid=%s' % (path, mchid or self._mchid, sub_mchid)
    else:
        if transaction_id:
            path = '/v3/pay/transactions/id/%s' % transaction_id
        elif out_trade_no:
            path = '/v3/pay/transactions/out-trade-no/%s' % out_trade_no
        else:
            raise Exception('transaction_id out_trade_no is not assigned.')
        path = '%s?mchid=%s' % (path, mchid or self._mchid)
    return self._core.request(path)


def refund(self,
           out_refund_no,
           amount,
           transaction_id=None,
           out_trade_no=None,
           reason=None,
           funds_account=None,
           goods_detail=None,
           notify_url=None,
           sub_mchid=None):
    """申请退款
    :param out_refund_no: 商户退款单号，示例值:'1217752501201407033233368018'
    :param amount: 金额信息，示例值:{'refund':888, 'total':888, 'currency':'CNY', 'refund_fee':100}
    :param transaction_id: 微信支付订单号，示例值:'1217752501201407033233368018'
    :param out_trade_no: 商户订单号，示例值:'1217752501201407033233368018'
    :param reason: 退款原因，示例值:'商品已售完'
    :param funds_account: 退款资金来源，示例值:'AVAILABLE'
    :param goods_detail: 退款商品，示例值:{'merchant_goods_id':'1217752501201407033233368018', 'wechatpay_goods_id':'1001', 'goods_name':'iPhone6s 16G', 'unit_price':528800, 'refund_amount':528800, 'refund_quantity':1}
    :param notify_url: 通知地址，示例值:'https://www.weixin.qq.com/wxpay/pay.php'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    params = {}
    if notify_url or self._notify_url:
        params.update({'notify_url': notify_url or self._notify_url})
    if out_refund_no:
        params.update({'out_refund_no': out_refund_no})
    else:
        raise Exception('out_refund_no is not assigned.')
    if amount:
        params.update({'amount': amount})
    else:
        raise Exception('amount is not assigned.')
    if transaction_id:
        params.update({'transaction_id': transaction_id})
    elif out_trade_no:
        params.update({'out_trade_no': out_trade_no})
    else:
        raise Exception('transaction_id is not assigned.')
    if reason:
        params.update({'reason': reason})
    if funds_account:
        params.update({'funds_account': funds_account})
    if goods_detail:
        params.update({'goods_detail': goods_detail})
    if self._partner_mode:
        if sub_mchid:
            params.update({'sub_mchid': sub_mchid})
        else:
            raise Exception('sub_mchid is not assigned.')
    path = '/v3/refund/domestic/refunds'
    return self._core.request(path, method=RequestType.POST, data=params)


def query_refund(self, out_refund_no, sub_mchid=None):
    """查询单笔退款
    :param out_refund_no: 商户退款单号，示例值:'1217752501201407033233368018'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    path = '/v3/refund/domestic/refunds/%s' % out_refund_no
    if self._partner_mode:
        if sub_mchid:
            path = '%s?sub_mchid=%s' % (path, sub_mchid)
        else:
            raise Exception('sub_mchid is not assigned.')
    return self._core.request(path)


def trade_bill(self, bill_date, bill_type='ALL', tar_type='GZIP', sub_mchid=None):
    """申请交易账单
    :param bill_date: 账单日期，示例值:'2019-06-11'
    :param bill_type: 账单类型, 默认值:'ALL'
    :param tar_type: 压缩类型，默认值:'GZIP'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    path = '/v3/bill/tradebill?bill_date=%s&bill_type=%s&tar_type=%s' % (bill_date, bill_type, tar_type)
    if self._partner_mode and sub_mchid:
        path = '%s&sub_mchid=%s' % (path, sub_mchid)
    return self._core.request(path)


def fundflow_bill(self, bill_date, account_type='BASIC', tar_type='GZIP'):
    """申请资金账单
    :param bill_date: 账单日期，示例值:'2019-06-11'
    :param account_type: 资金账户类型, 默认值:'BASIC'，基本账户, 可选:'OPERATION'，运营账户；'FEES'，手续费账户
    :param tar_type: 压缩类型，默认值:'GZIP'
    """
    if not bill_date:
        raise Exception('bill_date is not assigned.')
    path = '/v3/bill/fundflowbill?bill_date=%s&account_type=%s&tar_type=%s' % (bill_date, account_type, tar_type)
    return self._core.request(path)


def submch_fundflow_bill(self, sub_mchid, bill_date, account_type, algorithm='AEAD_AES_256_GCM', tar_type=None):
    """申请单个子商户资金账单
    :param sub_mchid: 子商户号，示例值:'19000000001'
    :param bill_date: 账单日期，格式YYYY-MM-DD，示例值:'2019-06-11'
    :param account_type: 资金账户类型，枚举值:'BASIC':基本账户，'OPERATION':运营账户，'FEES':手续费账户，示例值:'BASIC'
    :param algorithm: 加密算法，枚举值:'AEAD_AES_256_GCM':AEAD_AES_256_GCM加密算法
    :param tar_type: 压缩格式，枚举值:'GZIP':返回格式为.gzip的压缩包账单
    """
    path = '/v3/bill/sub-merchant-fundflowbill'
    if sub_mchid:
        path += '?sub_mchid=%s' % sub_mchid
    else:
        raise Exception('sub_mchid is not assigned.')
    if bill_date:
        path += '&bill_date=%s' % bill_date
    else:
        raise Exception('bill_date is not assigned.')
    if account_type:
        path += '&account_type=%s' % account_type
    else:
        raise Exception('account_type is not assigned.')
    if algorithm:
        path += '&algorithm=%s' % algorithm
    else:
        raise Exception('algorithm is not assigned.')
    if tar_type:
        path += '&tar_type=%s' % tar_type
    return self._core.request(path)


def download_bill(self, url):
    """下载账单
    :param url: 账单下载地址，示例值:'https://api.mch.weixin.qq.com/v3/billdownload/file?token=xxx'
    """
    path = url[len(self._core._gate_way):] if url.startswith(self._core._gate_way) else url
    return self._core.request(path, skip_verify=True)


def combine_pay(self,
                combine_out_trade_no,
                sub_orders,
                scene_info=None,
                combine_payer_info=None,
                time_start=None,
                time_expire=None,
                combine_appid=None,
                combine_mchid=None,
                notify_url=None,
                pay_type=None):
    """合单支付下单
    :param combine_out_trade_no: 合单商户订单号, 示例值:'P20150806125346'
    :param sub_orders: 子单信息，示例值:[{'mchid':'1900000109', 'attach':'深圳分店', 'amount':{'total_amount':100,'currency':'CNY'}, 'out_trade_no':'20150806125346', 'description':'腾讯充值中心-QQ会员充值', 'settle_info':{'profit_sharing':False, 'subsidy_amount':10}}]
    :param scene_info: 场景信息, 示例值:{'device_id':'POS1:123', 'payer_client_ip':'14.17.22.32'}
    :param combine_payer_info: 支付者, 示例值:{'openid':'oUpF8uMuAJO_M2pxb1Q9zNjWeS6o'}
    :param time_start: 交易起始时间，示例值:'2019-12-31T15:59:59+08:00'
    :param time_expire: 交易结束时间, 示例值:'2019-12-31T15:59:59+08:00'
    :param combine_appid: 合单商户appid, 示例值:'wxd678efh567hg6787'
    :param combine_mchid: 合单发起方商户号，示例值:'1900000109'
    :param notify_url: 通知地址, 示例值:'https://yourapp.com/notify'
    :param pay_type: 微信支付类型，示例值:WeChatPayType.JSAPI
    """
    params = {}
    params.update({'combine_appid': combine_appid or self._appid})
    params.update({'combine_mchid': combine_mchid or self._mchid})
    if not (notify_url or self._notify_url):
        raise Exception('notify_url is not assigned.')
    params.update({'notify_url': notify_url or self._notify_url})
    if combine_out_trade_no:
        params.update({'combine_out_trade_no': combine_out_trade_no})
    else:
        raise Exception('combine_out_trade_no is not assigned.')
    if sub_orders:
        params.update({'sub_orders': sub_orders})
    else:
        raise Exception('sub_orders is not assigned.')
    if scene_info:
        params.update({'scene_info': scene_info})
    if combine_payer_info:
        params.update({'combine_payer_info': combine_payer_info})
    if time_start:
        params.update({'time_start': time_start})
    if time_expire:
        params.update({'time_expire': time_expire})
    pay_type = pay_type or self._type
    if pay_type in [WeChatPayType.JSAPI, WeChatPayType.MINIPROG]:
        if not combine_payer_info:
            raise Exception('combine_payer_info is not assigned')
        path = '/v3/combine-transactions/jsapi'
    elif pay_type == WeChatPayType.APP:
        path = '/v3/combine-transactions/app'
    elif pay_type == WeChatPayType.H5:
        if not scene_info:
            raise Exception('scene_info is not assigned.')
        path = '/v3/combine-transactions/h5'
    elif pay_type == WeChatPayType.NATIVE:
        path = '/v3/combine-transactions/native'
    else:
        raise Exception('pay_type is not assigned.')
    return self._core.request(path, method=RequestType.POST, data=params)


def combine_query(self, combine_out_trade_no):
    """合单查询订单
    :param combine_out_trade_no: 合单商户订单号，示例值:P20150806125346
    """
    params = {}
    if not combine_out_trade_no:
        raise Exception('combine_out_trade_no is not assigned')
    else:
        params.update({'combine_out_trade_no': combine_out_trade_no})
    path = '/v3/combine-transactions/out-trade-no/%s' % combine_out_trade_no
    return self._core.request(path)


def combine_close(self, combine_out_trade_no, sub_orders, combine_appid=None):
    """合单关闭订单
    :param combine_out_trade_no: 合单商户订单号，示例值:'P20150806125346'
    :param sub_orders: 子单信息, 示例值:[{'mchid': '1900000109', 'out_trade_no': '20150806125346'}]
    :param combine_appid: 合单商户appid, 示例值:'wxd678efh567hg6787'
    """
    params = {}
    params.update({'combine_appid': combine_appid or self._appid})
    if not combine_out_trade_no:
        raise Exception('combine_out_trade_no is not assigned.')
    if not sub_orders:
        raise Exception('sub_orders is not assigned.')
    else:
        params.update({'sub_orders': sub_orders})
    path = '/v3/combine-transactions/out-trade-no/%s/close' % combine_out_trade_no
    return self._core.request(path, method=RequestType.POST, data=params)



# -*- coding: utf-8 -*-

from .type import RequestType


def apply4subject_submit(self, business_code, contact_info, subject_info, identification_info, channel_id=None, addition_info=None, ubo_info_list=[]):
    """（商户开户意愿）提交申请单
    :param business_code: 业务申请编号，示例值:'APPLYMENT_00000000001'
    :param contact_info: 联系人信息，示例值:{'name':'张三','id_card_number':'320311770706001','mobile':'13900000000'}
    :param subject_info: 主体信息，示例值:{'subject_type':'SUBJECT_TYPE_ENTERPRISE','business_license_info':{'license_copy':'demo-media-id','license_number':'123456789012345678','merchant_name':'腾讯科技有限公司','legal_person':'张三','company_address':'广东省深圳市南山区xx路xx号','licence_valid_date':'["1970-01-01","forever"]'}}
    :param business_info: 经营资料，示例值:{'merchant_shortname':'张三餐饮店','service_phone':'0758xxxxxx','sales_info':{'sales_scenes_type':['SALES_SCENES_STORE','SALES_SCENES_MP']}}
    :param identification_info: 法人身份信息，示例值:{'identification_type':'IDENTIFICATION_TYPE_IDCARD','identification_name':'张三','identification_number':'110220330044005500','identification_valid_date':'["1970-01-01","forever"]','identification_front_copy':'0P3ng6KTIW4-Q_l2FjKLZ...','identification_back_copy':'0P3ng6KTIW4-Q_l2FjKLZ...'}
    :param channel_id: 渠道商户号，示例值:'20001111'
    :param addition_info: 补充材料，示例值:{'confirm_mchid_list':['20001113']}
    :param ubo_info_list: 最终受益人信息列表，示例值:[{'ubo_id_doc_type':'IDENTIFICATION_TYPE_IDCARD','ubo_id_doc_name':'张三','ubo_id_doc_number':'110220330044005500'}]
    """
    params = {}
    if business_code:
        params.update({'business_code': business_code})
    else:
        raise Exception('business_code is not assigned.')
    if contact_info:
        params.update({'contact_info': contact_info})
    else:
        raise Exception('contact_info is not assigned.')
    if subject_info:
        params.update({'subject_info': subject_info})
    else:
        raise Exception('subject_info is not assigned.')
    if identification_info:
        params.update({'identification_info': identification_info})
    else:
        raise Exception('identification_info is not assigned')
    if channel_id:
        params.update({'channel_id': channel_id})
    if addition_info:
        params.update({'addition_info': addition_info})
    if ubo_info_list:
        params.update({'ubo_info_list': ubo_info_list})
    contact_name = params.get('contact_info').get('name')
    if contact_name:
        params['contact_info']['name'] = self._core.encrypt(contact_name)
    contact_mobile = params.get('contact_info').get('mobile')
    if contact_mobile:
        params['contact_info']['mobile'] = self._core.encrypt(contact_mobile)
    contact_number = params.get('contact_info').get('id_card_number')
    if contact_number:
        params['contact_info']['id_card_number'] = self._core.encrypt(contact_number)
    identification_name = params.get('identification_info').get('identification_name')
    if identification_name:
        params['identification_info']['identification_name'] = self._core.encrypt(identification_name)
    identification_number = params.get('identification_info').get('identification_number')
    if identification_number:
        params['identification_info']['identification_number'] = self._core.encrypt(identification_number)
    identification_address = params.get('identification_info').get('identification_address')
    if identification_address:
        params['identification_info']['identification_address'] = self._core.encrypt(identification_address)
    if params.get('ubo_info_list'):
        for ubo_info in params['ubo_info_list']:
            ubo_info['ubo_id_doc_name'] = self._core.encrypt(ubo_info['ubo_id_doc_name'])
            ubo_info['ubo_id_doc_number'] = self._core.encrypt(ubo_info['ubo_id_doc_number'])
            ubo_info['ubo_id_doc_address'] = self._core.encrypt(ubo_info['ubo_id_doc_address'])
    path = '/v3/apply4subject/applyment'
    return self._core.request(path, method=RequestType.POST, data=params, cipher_data=True)


def apply4subject_cancel(self, business_code=None, applyment_id=None):
    """（商户开户意愿）撤销申请单
    :param business_code: 业务申请编号，示例值:'2000001234567890'
    :param applyment_id: 申请单编号，示例值:2000001234567890
    """
    if business_code:
        path = '/v3/apply4subject/applyment/%s/cancel' % business_code
    elif applyment_id:
        path = '/v3/apply4subject/applyment/%s/cancel' % applyment_id
    else:
        raise Exception('business_code or applyment_id is not assigned.')
    return self._core.request(path)


def apply4subject_query(self, business_code=None, applyment_id=None):
    """（商户开户意愿）查询申请单审核结果
    :param business_code: 业务申请编号，示例值:'2000001234567890'
    :param applyment_id: 申请单编号，示例值:2000001234567890
    """
    if business_code:
        path = '/v3/apply4subject/applyment?business_code=%s' % business_code
    elif applyment_id:
        path = '/v3/apply4subject/applyment?applyment_id=%s' % applyment_id
    else:
        raise Exception('business_code or applyment_id is not assigned.')
    return self._core.request(path)


def apply4subject_state(self, sub_mchid):
    """（商户开户意愿）获取商户开户意愿确认状态
    :param sub_mchid: 特约商户号，示例值:'1511101111'
    """
    if sub_mchid:
        path = '/v3/apply4subject/applyment/merchants/%s/state' % sub_mchid
    else:
        raise Exception('sub_mchid is not assigned.')
    return self._core.request(path)



# -*- coding: utf-8 -*-

from .type import RequestType


def profitsharing_order(self, transaction_id, out_order_no, receivers, unfreeze_unsplit,
                        appid=None, sub_appid=None, sub_mchid=None):
    """请求分账
    :param transaction_id: 微信支付订单号，示例值:'4208450740201411110007820472'
    :param out_order_no: 商户分账单号，只能是数字、大小写字母_-|*@，示例值:'P20150806125346'
    :param receivers: 分账接收方列表，最多可有50个分账接收方，示例值:[{'type':'MERCHANT_ID', 'account':'86693852', 'amount':888, 'description':'分给商户A'}]
    :param unfreeze_unsplit: 是否解冻剩余未分资金，示例值:True, False
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    :param sub_appid: (服务商模式)子商户应用ID，示例值:'wxd678efh567hg6999'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    params = {}
    if transaction_id:
        params.update({'transaction_id': transaction_id})
    else:
        raise Exception('transaction_id is not assigned')
    if out_order_no:
        params.update({'out_order_no': out_order_no})
    else:
        raise Exception('out_order_no is not assigned')
    if isinstance(unfreeze_unsplit, bool):
        params.update({'unfreeze_unsplit': unfreeze_unsplit})
    else:
        raise Exception('unfreeze_unsplit is not assigned')
    if isinstance(receivers, list):
        params.update({'receivers': receivers})
    else:
        raise Exception('receivers is not assigned')
    cipher_data = False
    for receiver in params.get('receivers'):
        if receiver.get('name'):
            receiver['name'] = self._core.encrypt(receiver.get('name'))
            cipher_data = True
    params.update({'appid': appid or self._appid})
    if self._partner_mode:
        if sub_appid:
            params.update({'sub_appid': sub_appid})
        if sub_mchid:
            params.update({'sub_mchid': sub_mchid})
        else:
            raise Exception('sub_mchid is not assigned.')
    path = '/v3/profitsharing/orders'
    return self._core.request(path, method=RequestType.POST, data=params, cipher_data=cipher_data)


def profitsharing_order_query(self, transaction_id, out_order_no, sub_mchid=None):
    """查询分账结果
    :param transaction_id: 微信支付订单号，示例值:'4208450740201411110007820472'
    :param out_order_no: 商户分账单号，只能是数字、大小写字母_-|*@，示例值:'P20150806125346'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    if transaction_id and out_order_no:
        path = '/v3/profitsharing/orders/%s?transaction_id=%s' % (out_order_no, transaction_id)
    else:
        raise Exception('transaction_id or out_order_no is not assigned.')
    if self._partner_mode:
        if sub_mchid:
            path = '%s&sub_mchid=%s' % (path, sub_mchid)
        else:
            raise Exception('sub_mchid is not assigned.')
    return self._core.request(path)


def profitsharing_return(self, out_return_no, return_mchid, amount, description,
                         order_id=None, out_order_no=None, sub_mchid=None):
    """请求分账回退
    :param out_return_no: 商户回退单号，商户在自己后台生成的一个新的回退单号，在商户后台唯一，示例值:'R20190516001'
    :param return_mchid: 回退商户号，分账接口中的分账接收方商户号，示例值:'86693852'
    :param amount: 回退金额，单位为分，示例值:888
    :param description: 回退描述，分账回退的原因描述，示例值:'用户退款'
    :param order_id: 微信分账单号，与out_order_no参数二选一，示例值:'3008450740201411110007820472'
    :param out_order_no: 商户分账单号，只能是数字、大小写字母_-|*@，示例值:'P20150806125346'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    params = {}
    if order_id:
        params.update({'order_id': order_id})
    elif out_order_no:
        params.update({'out_order_no': out_order_no})
    else:
        raise Exception('order_id or out_order_no is not assigned.')
    if out_return_no:
        params.update({'out_return_no': out_return_no})
    else:
        raise Exception('out_return_no is not assigned')
    if return_mchid:
        params.update({'return_mchid': return_mchid})
    else:
        raise Exception('return_mchid is not assigned')
    if amount:
        params.update({'amount': amount})
    else:
        raise Exception('amount is not assigned')
    if description:
        params.update({'description': description})
    else:
        raise Exception('description is not assigned')
    if self._partner_mode:
        if sub_mchid:
            params.update({'sub_mchid': sub_mchid})
        else:
            raise Exception('sub_mchid is not assigned.')
    path = '/v3/profitsharing/return-orders'
    return self._core.request(path, method=RequestType.POST, data=params)


def profitsharing_return_query(self, out_order_no, out_return_no, sub_mchid=None):
    """查询分账回退结果
    :param out_order_no: 商户分账单号，只能是数字、大小写字母_-|*@，示例值:'P20150806125346'
    :param out_return_no: 商户回退单号，商户在自己后台生成的一个新的回退单号，在商户后台唯一，示例值:'R20190516001'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    if out_order_no and out_return_no:
        path = '/v3/profitsharing/return-orders/%s?&out_order_no=%s' % (out_return_no, out_order_no)
    else:
        raise Exception('out_order_no or out_return_no is not assigned')
    if self._partner_mode:
        if sub_mchid:
            path = '%s&sub_mchid=%s' % (path, sub_mchid)
        else:
            raise Exception('sub_mchid is not assigned.')
    return self._core.request(path)


def profitsharing_unfreeze(self, transaction_id, out_order_no, description, sub_mchid=None):
    """解冻剩余资金
    :param transaction_id: 微信支付订单号，示例值:'4208450740201411110007820472'
    :param out_order_no: 商户分账单号，只能是数字、大小写字母_-|*@，示例值:'P20150806125346'
    :param description: 分账描述，分账的原因描述，分账账单中需要体现，示例值:'解冻全部剩余资金'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    params = {}
    if transaction_id:
        params.update({'transaction_id': transaction_id})
    else:
        raise Exception('transaction_id is not assigned')
    if out_order_no:
        params.update({'out_order_no': out_order_no})
    else:
        raise Exception('out_order_no is not assigned')
    if description:
        params.update({'description': description})
    else:
        raise Exception('description is not assigned')
    if self._partner_mode:
        if sub_mchid:
            params.update({'sub_mchid': sub_mchid})
        else:
            raise Exception('sub_mchid is not assigned.')
    path = '/v3/profitsharing/orders/unfreeze'
    return self._core.request(path, method=RequestType.POST, data=params)


def profitsharing_amount_query(self, transaction_id):
    """查询剩余待分金额
    :param transaction_id: 微信支付订单号，示例值:'4208450740201411110007820472'
    """
    if transaction_id:
        path = '/v3/profitsharing/transactions/%s/amounts' % transaction_id
    else:
        raise Exception('transaction_id is not assigned')
    return self._core.request(path)


def profitsharing_config_query(self, sub_mchid):
    """查询最大分账比例
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    if sub_mchid:
        path = '/v3/profitsharing/merchant-configs/%s' % sub_mchid
    else:
        raise Exception('sub_mchid is not assigned')
    return self._core.request(path)


def profitsharing_add_receiver(self, account_type, account, relation_type, name=None,
                               custom_relation=None, appid=None, sub_appid=None, sub_mchid=None):
    """添加分账接收方
    :param account_type: 分账接收方类型，枚举值:'MERCHANT_ID':商户ID，'PERSONAL_OPENID':个人openid
    :param account: 分账接收方账号，类型是'MERCHANT_ID'时，是商户号，类型是'PERSONAL_OPENID'时，是个人openid，示例值:'86693852'
    :param relation_type:与分账方的关系类型，枚举值:'STORE':门店，'STAFF':员工，'STORE_OWNER':店主，
                            'PARTNER':合作伙伴，'HEADQUARTER':总部，'BRAND':品牌方，'DISTRIBUTOR':分销商，
                            'USER':用户，'SUPPLIER': 供应商，'CUSTOM':自定义，示例值:'STORE'
    :param name: 分账个人接收方姓名，分账接收方类型是'MERCHANT_ID'时，是商户全称（必传），当商户是小微商户或个体户时，是开户人姓名，
                            分账接收方类型是'PERSONAL_OPENID'时，是个人姓名
    :param custom_relation: 自定义的分账关系，子商户与接收方具体的关系，本字段最多10个字。当字段'relation_type'的值为'CUSTOM'时，本字段必填;
                            当字段'relation_type'的值不为'CUSTOM'时，本字段无需填写。示例值:'代理商'
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    :param sub_appid: (服务商模式)子商户应用ID，示例值:'wxd678efh567hg6999'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    params = {}
    if account_type:
        params.update({'type': account_type})
    else:
        raise Exception('account_type is not assigned')
    if account:
        params.update({'account': account})
    else:
        raise Exception('account is not assigned')
    if relation_type:
        params.update({'relation_type': relation_type})
    else:
        raise Exception('relation_type is not assigned')
    cipher_data = False
    if name:
        params.update({'name': self._core.encrypt(name)})
        cipher_data = True
    if relation_type == 'CUSTOM':
        if custom_relation:
            params.update({'custom_relation': custom_relation})
        else:
            raise Exception('custom_relation is not assigned')
    params.update({'appid': appid or self._appid})
    if self._partner_mode:
        if sub_appid:
            params.update({'sub_appid': sub_appid})
        if sub_mchid:
            params.update({'sub_mchid': sub_mchid})
        else:
            raise Exception('sub_mchid is not assigned.')
    path = '/v3/profitsharing/receivers/add'
    return self._core.request(path, method=RequestType.POST, data=params, cipher_data=cipher_data)


def profitsharing_delete_receiver(self, account_type, account, appid=None, sub_appid=None, sub_mchid=None):
    """删除分账接收方
    :param account_type: 分账接收方类型，枚举值:'MERCHANT_ID':商户ID，'PERSONAL_OPENID':个人openid
    :param account: 分账接收方账号，类型是'MERCHANT_ID'时，是商户号，类型是'PERSONAL_OPENID'时，是个人openid，示例值:'86693852'
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    :param sub_appid: (服务商模式)子商户应用ID，示例值:'wxd678efh567hg6999'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    params = {}
    if account_type:
        params.update({'type': account_type})
    else:
        raise Exception('account_type is not assigned')
    if account:
        params.update({'account': account})
    else:
        raise Exception('account is not assigned')
    params.update({'appid': appid or self._appid})
    if self._partner_mode:
        if sub_appid:
            params.update({'sub_appid': sub_appid})
        if sub_mchid:
            params.update({'sub_mchid': sub_mchid})
        else:
            raise Exception('sub_mchid is not assigned.')
    path = '/v3/profitsharing/receivers/delete'
    return self._core.request(path, method=RequestType.POST, data=params)


def profitsharing_bill(self, bill_date, tar_type='GZIP', sub_mchid=None):
    """申请分账账单
    :param bill_date: 账单日期，格式'YYYY-MM-DD'，仅支持三个月内的账单下载申请。示例值:'2019-06-11'
    :param tar_type: 压缩类型，默认值:'GZIP'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    path = '/v3/profitsharing/bills?bill_date=%s&tar_type=%s' % (bill_date, tar_type)
    if self._partner_mode and sub_mchid:
        path = '%s&sub_mchid=%s' % (path, sub_mchid)
    return self._core.request(path)


def brand_profitsharing_order(self, brand_mchid, sub_mchid, transaction_id, out_order_no, receivers,
                              finish, appid=None, sub_appid=None):
    """连锁品牌请求分账
    :param brand_mchid: 品牌主商户号，示例值:'1900000108'
    :param sub_mchid: 子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    :param transaction_id: 微信支付订单号，示例值:'4208450740201411110007820472'
    :param out_order_no: 商户分账单号，只能是数字、大小写字母_-|*@，示例值:'P20150806125346'
    :param receivers: 分账接收方列表，最多可有50个分账接收方，示例值:{{'type':'MERCHANT_ID', 'account':'86693852', 'amount':888, 'description':'分给商户A'}}
    :param finish: 是否完成分账，示例值:True, False
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    :param sub_appid: 子商户应用ID，示例值:'wxd678efh567hg6999'
    """
    params = {}
    if brand_mchid:
        params.update({'brand_mchid': brand_mchid})
    else:
        raise Exception('brand_mchid is not assigned')
    if sub_mchid:
        params.update({'sub_mchid': sub_mchid})
    else:
        raise Exception('sub_mchid is not assigned.')
    if transaction_id:
        params.update({'transaction_id': transaction_id})
    else:
        raise Exception('transaction_id is not assigned')
    if out_order_no:
        params.update({'out_order_no': out_order_no})
    else:
        raise Exception('out_order_no is not assigned')
    if receivers:
        params.update({'receivers': receivers})
    else:
        raise Exception('receivers is not assigned')
    if isinstance(finish, bool):
        params.update({'finish': finish})
    else:
        raise Exception('finish is not assigned')
    params.update({'appid': appid or self._appid})
    if sub_appid:
        params.update({'sub_appid': sub_appid})
    path = '/v3/brand/profitsharing/orders'
    return self._core.request(path, method=RequestType.POST, data=params)


def brand_profitsharing_order_query(self, transaction_id, out_order_no, sub_mchid):
    """查询连锁品牌分账结果
    :param transaction_id: 微信支付订单号，示例值:'4208450740201411110007820472'
    :param out_order_no: 商户分账单号，只能是数字、大小写字母_-|*@，示例值:'P20150806125346'
    :param sub_mchid: 子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    if sub_mchid:
        path = '/v3/brand/profitsharing/orders?sub_mchid=%s' % sub_mchid
    else:
        raise Exception('sub_mchid is not assigned.')
    if transaction_id and out_order_no:
        path = '%s&transaction_id=%s&out_order_no=%s' % (transaction_id, out_order_no)
    else:
        raise Exception('transaction_id or out_order_no is not assigned.')
    return self._core.request(path)


def brand_profitsharing_return(self, sub_mchid, out_return_no, return_mchid, amount,
                               description, order_id=None, out_order_no=None,):
    """请求连锁品牌分账回退
    :param sub_mchid: 子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    :param out_return_no: 商户回退单号，商户在自己后台生成的一个新的回退单号，在商户后台唯一，示例值:'R20190516001'
    :param return_mchid: 回退商户号，分账接口中的分账接收方商户号，示例值:'86693852'
    :param amount: 回退金额，单位为分，示例值:888
    :param description: 回退描述，分账回退的原因描述，示例值:'用户退款'
    :param order_id: 微信分账单号，与out_order_no参数二选一，示例值:'3008450740201411110007820472'
    :param out_order_no: 商户分账单号，只能是数字、大小写字母_-|*@，示例值:'P20150806125346'
    """
    params = {}
    if not (order_id and out_order_no):
        raise Exception('order_id or out_order_no is not assigned')
    if order_id:
        params.update({'order_id': order_id})
    elif out_order_no:
        params.update({'out_order_no': out_order_no})
    else:
        raise Exception('order_id or out_order_no is not assigned.')
    if out_return_no:
        params.update({'out_return_no': out_return_no})
    else:
        raise Exception('out_return_no is not assigned')
    if return_mchid:
        params.update({'return_mchid': return_mchid})
    else:
        raise Exception('return_mchid is not assigned')
    if amount:
        params.update({'amount': amount})
    else:
        raise Exception('amount is not assigned')
    if description:
        params.update({'description': description})
    else:
        raise Exception('description is not assigned')
    if sub_mchid:
        params.update({'sub_mchid': sub_mchid})
    else:
        raise Exception('sub_mchid is not assigned.')
    path = '/v3/brand/profitsharing/returnorders'
    return self._core.request(path, method=RequestType.POST, data=params)


def brand_profitsharing_return_query(self, sub_mchid, out_return_no, order_id=None, out_order_no=None):
    """查询连锁品牌分账回退结果
    :param sub_mchid: 子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    :param out_return_no: 商户回退单号，商户在自己后台生成的一个新的回退单号，在商户后台唯一，示例值:'R20190516001'
    :param order_id: 微信分账单号，与out_order_no参数二选一，示例值:'3008450740201411110007820472'
    :param out_order_no: 商户分账单号，只能是数字、大小写字母_-|*@，示例值:'P20150806125346'
    """
    if sub_mchid:
        path = '/v3/brand/profitsharing/returnorders?sub_mchid=%s' % sub_mchid
    else:
        raise Exception('sub_mchid is not assigned.')
    if out_return_no:
        path = '%s&out_return_no=%s' % (path, out_return_no)
    else:
        raise Exception('out_return_no is not assigned')
    if order_id:
        path = '%s&order_id=%s' % (path, order_id)
    elif out_order_no:
        path = '%s&out_order_no=%s' % (path, out_order_no)
    else:
        raise Exception('order_id or out_order_no is not assigned.')
    return self._core.request(path)


def brand_profitsharing_unfreeze(self, sub_mchid, transaction_id, out_order_no, description):
    """完结连锁品牌分账
    :param sub_mchid: 子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    :param transaction_id: 微信支付订单号，示例值:'4208450740201411110007820472'
    :param out_order_no: 商户分账单号，只能是数字、大小写字母_-|*@，示例值:'P20150806125346'
    :param description: 分账描述，分账的原因描述，分账账单中需要体现，示例值:'解冻全部剩余资金'
    """
    params = {}
    if sub_mchid:
        params.update({'sub_mchid': sub_mchid})
    else:
        raise Exception('sub_mchid is not assigned.')
    if transaction_id:
        params.update({'transaction_id': transaction_id})
    else:
        raise Exception('transaction_id is not assigned')
    if out_order_no:
        params.update({'out_order_no': out_order_no})
    else:
        raise Exception('out_order_no is not assigned')
    if description:
        params.update({'description': description})
    else:
        raise Exception('description is not assigned')
    path = '/v3/brand/profitsharing/finish-order'
    return self._core.request(path, method=RequestType.POST, data=params)


def brand_profitsharing_amount_query(self, transaction_id):
    """查询连锁品牌分账剩余待分金额
    :param transaction_id: 微信支付订单号，示例值:'4208450740201411110007820472'
    """
    if transaction_id:
        path = '/v3/brand/profitsharing/orders/%s/amounts' % transaction_id
    else:
        raise Exception('transaction_id is not assigned')
    return self._core.request(path)


def brand_profitsharing_config_query(self, brand_mchid):
    """查询连锁品牌分账最大分账比例
    :param brand_mchid: 品牌商户号，示例值:'1900000108'
    """
    if brand_mchid:
        path = '/v3/brand/profitsharing/brand-configs/%s' % brand_mchid
    else:
        raise Exception('brand_mchid is not assigned')
    return self._core.request(path)


def brand_profitsharing_add_receiver(self, brand_mchid, account_type, account, relation_type,
                                     name=None, appid=None, sub_appid=None):
    """添加分账接收方
    :param brand_mchid: 品牌商户号，示例值:'1900000108'
    :param account_type: 分账接收方类型，枚举值:'MERCHANT_ID':商户ID，'PERSONAL_OPENID':个人openid
    :param account: 分账接收方账号，类型是'MERCHANT_ID'时，是商户号，类型是'PERSONAL_OPENID'时，是个人openid，示例值:'86693852'
    :param relation_type:与分账方的关系类型，枚举值:'STORE':门店，'STAFF':员工，'STORE_OWNER':店主，
                            'PARTNER':合作伙伴，'HEADQUARTER':总部，'BRAND':品牌方，'DISTRIBUTOR':分销商，
                            'USER':用户，'SUPPLIER': 供应商，'CUSTOM':自定义，示例值:'STORE'
    :param name: 分账个人接收方姓名，分账接收方类型是'MERCHANT_ID'时，是商户全称（必传），当商户是小微商户或个体户时，是开户人姓名，
                            分账接收方类型是'PERSONAL_OPENID'时，是个人姓名
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    :param sub_appid: 子商户应用ID，示例值:'wxd678efh567hg6999'
    """
    params = {}
    if brand_mchid:
        params.update({'brand_mchid': brand_mchid})
    else:
        raise Exception('brand_mchid is not assigned.')
    if account_type:
        params.update({'type': account_type})
    else:
        raise Exception('account_type is not assigned')
    if account:
        params.update({'account': account})
    else:
        raise Exception('account is not assigned')
    if relation_type:
        params.update({'relation_type': relation_type})
    else:
        raise Exception('relation_type is not assigned')
    cipher_data = False
    if name:
        params.update({'name': self._core.encrypt(name)})
        cipher_data = True
    params.update({'appid': appid or self._appid})
    if sub_appid:
        params.update({'sub_appid': sub_appid})
    path = '/v3/brand/profitsharing/receivers/add'
    return self._core.request(path, method=RequestType.POST, data=params, cipher_data=cipher_data)


def brand_profitsharing_delete_receiver(self, brand_mchid, account_type, account, appid=None, sub_appid=None):
    """删除连锁品牌分账接收方
    :param brand_mchid: 品牌商户号，示例值:'1900000108'
    :param account_type: 分账接收方类型，枚举值:'MERCHANT_ID':商户ID，'PERSONAL_OPENID':个人openid
    :param account: 分账接收方账号，类型是'MERCHANT_ID'时，是商户号，类型是'PERSONAL_OPENID'时，是个人openid，示例值:'86693852'
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    :param sub_appid: (服务商模式)子商户应用ID，示例值:'wxd678efh567hg6999'
    """
    params = {}
    if brand_mchid:
        params.update({'brand_mchid': brand_mchid})
    else:
        raise Exception('brand_mchid is not assigned.')
    if account_type:
        params.update({'type': account_type})
    else:
        raise Exception('account_type is not assigned')
    if account:
        params.update({'account': account})
    else:
        raise Exception('account is not assigned')
    params.update({'appid': appid or self._appid})
    if sub_appid:
        params.update({'sub_appid': sub_appid})
    path = '/v3/profitsharing/receivers/delete'
    return self._core.request(path, method=RequestType.POST, data=params)



# -*- coding: utf-8 -*-

from .type import RequestType


def parking_service_find(self, plate_number, plate_color, openid, sub_mchid=None):
    """查询车牌服务开通信息
    :param plate_number: 车牌号，示例值:'粤B888888'
    :param plate_color: 车牌颜色，车牌颜色，枚举值:BLUE:蓝色，GREEN:绿色，YELLOW:黄色，BLACK:黑色，WHITE:白色，LIMEGREEN:黄绿色
    :param openid: 用户标识，示例值:'oUpF8uMuAJOM2pxb1Q'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    path = '/v3/vehicle/parking/services/find?appid=%s' % self._appid
    if plate_number:
        path = '%s&plate_number=%s' % (path, plate_number)
    else:
        raise Exception('plate_number is not assigned.')
    if plate_color:
        path = '%s&plate_color=%s' % (path, plate_color)
    else:
        raise Exception('plate_color is not assigned.')
    if openid:
        path = '%s&openid=%s' % (path, openid)
    else:
        raise Exception('openid is not assigned.')
    if self._partner_mode:
        if sub_mchid:
            path = '%s&sub_mchid=%s' % (path, sub_mchid)
        else:
            raise Exception('sub_mchid is not assigned.')
    return self._core.request(path)


def parking_enter(self, out_parking_no, plate_number, plate_color, start_time, parking_name, free_duration, notify_url=None, sub_mchid=None):
    """创建停车入场
    :param out_parking_no: 商户入场id，商户侧入场标识id，在同一个商户号下唯一，示例值:'1231243'
    :param plate_number: 车牌号，示例值:'粤B888888'
    :param plate_color: 车牌颜色，车牌颜色，枚举值:BLUE:蓝色，GREEN:绿色，YELLOW:黄色，BLACK:黑色，WHITE:白色，LIMEGREEN:黄绿色
    :param notify_url: 回调通知url，接受入场状态变更回调通知的url，只接受https，示例值:https://yoursite.com/wxpay.html
    :param start_time: 入场时间，示例值:'2017-08-26T10:43:39+08:00'
    :param parking_name: 停车场名称，示例值:'欢乐海岸停车场'
    :param free_duration: 免费时长，单位为秒，示例值:3600
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    params = {}
    if out_parking_no:
        params.update({'out_parking_no': out_parking_no})
    else:
        raise Exception('out_parking_no is not assigned')
    if plate_number:
        params.update({'plate_number': plate_number})
    else:
        raise Exception('plate_number is not assigned')
    if plate_color:
        params.update({'plate_color': plate_color})
    else:
        raise Exception('plate_color is not assigned')
    if not (notify_url or self._notify_url):
        raise Exception('notify_url is not assigned.')
    params.update({'notify_url': notify_url or self._notify_url})
    if start_time:
        params.update({'start_time': start_time})
    else:
        raise Exception('start_time is not assigned')
    if parking_name:
        params.update({'parking_name': parking_name})
    else:
        raise Exception('parking_name is not assigned')
    if free_duration:
        params.update({'free_duration': free_duration})
    else:
        raise Exception('free_duration is not assigned')
    if self._partner_mode:
        if sub_mchid:
            params.update({'sub_mchid': sub_mchid})
        else:
            raise Exception('sub_mchid is not assigned.')
    path = '/v3/vehicle/parking/parkings'
    return self._core.request(path, method=RequestType.POST, data=params)


def parking_order(self, description, out_trade_no, total, parking_id, plate_number, plate_color, start_time,
                  end_time, parking_name, charging_duration, device_id, trade_scene='PARKING', profit_sharing='N',
                  currency='CNY', attach=None, goods_tag=None, notify_url=None, appid=None, sub_appid=None, sub_mchid=None):
    """停车扣费受理
    :param description: 服务描述，商户自定义字段，用于交易账单中对扣费服务的描述。示例值:'停车场扣费'
    :param out_trade_no: 商户订单号，商户系统内部订单号，只能是数字、大小写字母，且在同一个商户号下唯一，示例值:'20150806125346'
    :param notify_url: 回调通知url，只接受https，示例值:'https://yoursite.com/wxpay.html'
    :param total: 订单总金额，单位为分，只能为整数，示例值:888
    :param parking_id: 停车入场id，通过入场通知接口获取的入场id，示例值:'5K8264ILTKCH16CQ250'
    :param plate_number: 车牌号，仅包括省份+车牌，不包括特殊字符。示例值:'粤B888888'
    :param plate_color: 车牌颜色，枚举值:BLUE:蓝色，GREEN:绿色，YELLOW:黄色，BLACK:黑色，WHITE:白色，LIMEGREEN:黄绿色，示例值:BLUE
    :param start_time: 入场时间，示例值:'2017-08-26T10:43:39+08:00'
    :param end_time: 出场时间，示例值:'2017-08-26T10:43:39+08:00'
    :param parking_name: 停车场名称，示例值:'欢乐海岸停车场'
    :param charging_duration: 计费时长，单位为秒，示例值:3600
    :param device_id: 停车场设备id，示例值:'12313'
    :param trade_scene: 交易场景值，目前支持'PARKING':车场停车场景
    :param profit_sharing: 分账标识，枚举值:'Y':是，需要分账，'N':否，不分账，字母要求大写，不传默认不分账。
    :param currency: 货币类型，目前只支持人民币:'CNY'
    :param attach: 附加数据，在查询API和支付通知中原样返回，可作为自定义参数使用，示例值:'深圳分店'
    :param goods_tag: 订单优惠标记，代金券或立减优惠功能的参数，示例值:WXG
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    :param sub_appid: (服务商模式)子商户应用ID，示例值:'wxd678efh567hg6999'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    params = {}
    amount = {}
    parking_info = {}
    params.update({'appid': appid or self._appid})
    if description:
        params.update({'description': description})
    else:
        raise Exception('description is not assigned')
    if out_trade_no:
        params.update({'out_trade_no': out_trade_no})
    else:
        raise Exception('out_trade_no is not assigned')
    params.update({'notify_url': notify_url or self._notify_url})
    if total:
        amount.update({'total': total})
    else:
        raise Exception('total is not assigned')
    if parking_id:
        parking_info.update({'parking_id': parking_id})
    else:
        raise Exception('parking_id is not assigned')
    if plate_number:
        parking_info.update({'plate_number': plate_number})
    else:
        raise Exception('plate_number is not assigned')
    if plate_color:
        parking_info.update({'plate_color': plate_color})
    else:
        raise Exception('plate_color is not assigned')
    if start_time:
        parking_info.update({'start_time': start_time})
    else:
        raise Exception('start_time is not assigned')
    if end_time:
        parking_info.update({'end_time': end_time})
    else:
        raise Exception('end_time is not assigned')
    if parking_name:
        parking_info.update({'parking_name': parking_name})
    else:
        raise Exception('parking_name is not assigned')
    if charging_duration:
        parking_info.update({'charging_duration': charging_duration})
    else:
        raise Exception('charging_duration is not assigned')
    if device_id:
        parking_info.update({'device_id': device_id})
    else:
        raise Exception('device_id is not assigned')
    if trade_scene:
        params.update({'trade_scene': trade_scene})
    else:
        raise Exception('trade_scene is not assigned')
    if profit_sharing:
        params.update({'profit_sharing': profit_sharing})
    else:
        raise Exception('profit_sharing is not assigned')
    if currency:
        amount.update({'currency': currency})
    else:
        raise Exception('currency is not assigned')
    if attach:
        params.update({'attach': attach})
    if goods_tag:
        params.update({'goods_tag': goods_tag})
    params.update({'amount': amount})
    params.update({'parking_info': parking_info})
    if self._partner_mode:
        if sub_appid:
            params.update({'sub_appid': sub_appid})
        else:
            raise Exception('sub_appid is not assigned.')
        if sub_mchid:
            params.update({'sub_mchid': sub_mchid})
        else:
            raise Exception('sub_mchid is not assigned.')
    path = '/v3/vehicle/transactions/parking'
    return self._core.request(path, method=RequestType.POST, data=params)


def parking_order_query(self, out_trade_no, sub_mchid=None):
    """停车扣费订单查询
    :param out_trade_no: 商户订单号，商户系统内部订单号，只能是数字、大小写字母，且在同一个商户号下唯一，示例值:'20150806125346'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    if out_trade_no:
        path = '/v3/vehicle/transactions/out-trade-no/%s' % out_trade_no
    else:
        raise Exception('out_trade_no is not assigned')
    if self._partner_mode:
        if sub_mchid:
            path = '%s?sub_mchid=%s' % (path, sub_mchid)
        else:
            raise Exception('sub_mchid is not assigned.')
    return self._core.request(path)



# -*- coding: utf-8 -*-

import os

from .media import _media_upload
from .type import RequestType
from .utils import sha256


def marketing_image_upload(self, filepath, filename=None):
    """图片上传(营销专用)
    :param filepath: 图片文件路径
    :param filename: 文件名称，未指定则从filepath参数中截取
    """
    return _media_upload(self, filepath, filename, '/v3/marketing/favor/media/image-upload')


def marketing_card_send(self, card_id, openid, out_request_no, send_time, appid=None):
    """发放消费卡
    :card_id: 消费卡ID。示例值:'pIJMr5MMiIkO_93VtPyIiEk2DZ4w'
    :openid: 用户openid，待发卡用户的openid。示例值:'obLatjhnqgy2syxrXVM3MJirbkdI'
    :out_request_no: 商户单据号。示例值:'oTYhjfdsahnssddj_0136'
    :send_time: 请求发卡时间，单次请求发卡时间，消费卡在商户系统的实际发放时间，为东八区标准时间（UTC+8）。示例值:'2019-12-31T13:29:35.120+08:00'
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    """
    params = {}
    if card_id:
        path = '/v3/marketing/busifavor/coupons/%s/send' % card_id
    else:
        raise Exception('card_id is not assigned.')
    if openid:
        params.update({'openid': openid})
    else:
        raise Exception('openid is not assigned.')
    if out_request_no:
        params.update({'out_request_no': out_request_no})
    else:
        raise Exception('out_request_no is not assigned.')
    if send_time:
        params.update({'send_time': send_time})
    else:
        raise Exception('send_time is not assigned.')
    params.update({'appid': appid or self._appid})
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_partnership_build(self, idempotency_key, partner_type, business_type, partner_appid=None,
                                partner_merchant_id=None, stock_id=None):
    """建立合作关系
    :idempotency_key: 业务请求幂等值，商户侧需保持唯一性，可包含英文字母，数字，｜，_，*，-等内容，不允许出现其他不合法符号。示例值:'12345'
    :partner_type: 合作方类别，枚举值:'APPID':合作方为APPID，'MERCHANT':合作方为商户。示例值:'APPID'
    :business_type: 授权业务类别，枚举值:'FAVOR_STOCK':代金券批次，'BUSIFAVOR_STOCK':商家券批次。示例值:'FAVOR_STOCK'
    :partner_appid: 合作方APPID，合作方类别为APPID时必填。示例值:'wx4e1916a585d1f4e9'
    :partner_merchant_id: 合作方商户ID，合作方类别为MERCHANT时必填。特殊规则:最小字符长度为8。示例值:'2480029552'
    :stock_id: 授权批次ID，授权业务类别为商家券批次或代金券批次时，此参数必填。示例值:'2433405'
    """
    headers = {}
    if idempotency_key:
        headers.update({'Idempotency-Key': idempotency_key})
    else:
        raise Exception('idempotency_key is not assigned.')
    params = {}
    if partner_type == 'APPID' and partner_appid:
        params.update({'partner': {'type': partner_type, 'appid': partner_appid}})
    elif partner_type == 'MERCHANT' and partner_merchant_id:
        params.update({'partner': {'type': partner_type, 'merchant_id': partner_merchant_id}})
    else:
        raise Exception('invalid value in partner_type/partner_appid/partner_merchant_id')
    if business_type not in ['FAVOR_STOCK', 'BUSIFAVOR_STOCK'] or not stock_id:
        raise Exception('invalid value in bussiness_type/stock_id.')
    params.update({'authorized_data': {'bussiness_type': business_type, 'stock_id': stock_id}})
    path = '/v3/marketing/partnerships/build'
    return self._core.request(path, method=RequestType.POST, data=params, headers=headers)


def marketing_partnership_query(self, business_type, stock_id, partner_type=None, partner_appid=None,
                                partner_merchant_id=None, limit=20, offset=None):
    """查询合作关系列表
    :business_type: 授权业务类别，枚举值:'FAVOR_STOCK':代金券批次，'BUSIFAVOR_STOCK':商家券批次。示例值:'FAVOR_STOCK'
    :stock_id: 授权批次ID，授权业务类别为商家券批次或代金券批次时，此参数必填。示例值:'2433405'
    :partner_type: 合作方类别，枚举值:'APPID':合作方为APPID，'MERCHANT':合作方为商户。示例值:'APPID'
    :partner_appid: 合作方APPID，合作方类别为APPID时必填。示例值:'wx4e1916a585d1f4e9'
    :partner_merchant_id: 合作方商户ID，合作方类别为MERCHANT时必填。特殊规则:最小字符长度为8。示例值:'2480029552'
    :limit: 分页大小，最大50。不传默认为20。示例值:5
    :offset: 分页页码，页码从0开始。示例值:10
    """
    path = '/v3/marketing/partnerships?'
    if business_type not in ['FAVOR_STOCK', 'BUSIFAVOR_STOCK'] or not stock_id:
        raise Exception('invalid value in bussiness_type/stock_id.')
    path = '%sauthorized_data={"business_type":"%s","stock_id":"%s"}' % (path, business_type, stock_id)
    if partner_type == 'APPID' and partner_appid:
        path = '%s&partner={"type":"%s","appid":"%s"}' % (path, partner_type, partner_appid)
    elif partner_type == 'MERCHANT' and partner_merchant_id:
        path = '%s&partner={"type":"%s","merchant_id":"%s"}' % (path, partner_type, partner_merchant_id)
    if limit in range(0, 51):
        path = '%s&limit=%s' % (path, limit)
    if offset:
        path = '%s&offset=%s' % (path, offset)
    return self._core.request(path)


def marketing_paygift_activity_create(self, activity_base_info, award_send_rule, advanced_setting=None):
    """创建全场满额送活动
    :param activity_base_info: 活动基本信息
    :param award_send_rule: 活动奖品发放规则
    :param advanced_setting: 活动高级设置
    """
    params = {}
    if not activity_base_info or not award_send_rule:
        raise Exception('activity_base_info or award_send_rule is not assigned.')
    params.update({'activity_base_info': activity_base_info})
    params.update({'award_send_rule': award_send_rule})
    if advanced_setting:
        params.update({'advanced_setting': advanced_setting})
    path = '/v3/marketing/paygiftactivity/unique-threshold-activity'
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_paygift_activity_detail(self, activity_id):
    """查询活动详情接口
    :param activity_id: 活动id，示例值:'10028001'
    """
    if activity_id:
        path = '/v3/marketing/paygiftactivity/activities/%s' % activity_id
    else:
        raise Exception('activity_id is not assigned.')
    return self._core.request(path)


def marketing_paygift_merchants_list(self, activity_id, offset=0, limit=20):
    """查询活动发券商户号
    :param activity_id: 活动id，示例值:'10028001'
    :param offset:分页页码，页面从0开始。示例值:1
    :param limit: 分页大小，限制分页最大数据条目。示例值:20
    """
    if activity_id:
        path = '/v3/marketing/paygiftactivity/activities/%s/merchants' % activity_id
    else:
        raise Exception('activity_id is not assigned.')
    path = '%s?offset=%s&limit=%s' % (path, offset, limit)
    return self._core.request(path)


def marketing_paygift_goods_list(self, activity_id, offset=0, limit=20):
    """查询活动指定商品列表
    :param activity_id: 活动id，示例值:'10028001'
    :param offset:分页页码，页面从0开始。示例值:1
    :param limit: 分页大小，限制分页最大数据条目。示例值:20
    """
    if activity_id:
        path = '/v3/marketing/paygiftactivity/activities/%s/goods' % activity_id
    else:
        raise Exception('activity_id is not assigned.')
    path = '%s?offset=%s&limit=%s' % (path, offset, limit)
    return self._core.request(path)


def marketing_paygift_activity_terminate(self, activity_id):
    """终止活动
    :param activity_id: 活动id，示例值:'10028001'
    """
    if activity_id:
        path = '/v3/marketing/paygiftactivity/activities/%s/terminate' % activity_id
    else:
        raise Exception('activity_id is not assigned.')
    return self._core.request(path, method=RequestType.POST)


def marketing_paygift_merchant_add(self, activity_id, add_request_no, merchant_id_list=[]):
    """新增活动发券商户号
    :param activity_id: 活动id，示例值:'10028001'
    :param add_request_no: 请求业务单据号，商户添加发券商户号的凭据号，商户侧需保持唯一性。示例值:'100002322019090134234sfdf'
    :param merchant_id_list: 发券商户号，新增到活动中的发券商户号列表，特殊规则:最小字符长度为8，最大为15，条目个数限制:[1，500]。示例值:["10000022"，"10000023"]
    """
    if activity_id:
        path = '/v3/marketing/paygiftactivity/activities/%s/merchants/add' % activity_id
    else:
        raise Exception('activity_id is not assigned.')
    params = {}
    if add_request_no:
        params.update({'add_request_no': add_request_no})
    else:
        raise Exception('add_request_no is not assigned.')
    if merchant_id_list:
        params.update({'merchant_id_list': merchant_id_list})
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_paygift_activity_list(self, offset=0, limit=20, activity_name=None, activity_status=None, award_type=None):
    """获取支付有礼活动列表
    :param offset:分页页码，页面从0开始。示例值:1
    :param limit: 分页大小，限制分页最大数据条目。示例值:20
    :param activity_name: 活动名称，支持模糊搜索。示例值:'良品铺子回馈活动'
    :param activity_status: 活动状态，枚举值:'ACT_STATUS_UNKNOWN':状态未知，'CREATE_ACT_STATUS':已创建，'ONGOING_ACT_STATUS':运行中，'TERMINATE_ACT_STATUS':已终止，
                            'STOP_ACT_STATUS':已暂停，'OVER_TIME_ACT_STATUS':已过期，'CREATE_ACT_FAILED':创建活动失败。示例值:'CREATE_ACT_STATUS'
    :param award_type: 奖品类型，暂时只支持商家券。'BUSIFAVOR':商家券。示例值:'BUSIFAVOR'
    """
    params = {}
    params.update({'offset': offset})
    params.update({'limit': limit})
    if activity_name:
        params.update({'activity_name': activity_name})
    if activity_status:
        params.update({'activity_status': activity_status})
    if award_type:
        params.update({'award_type': award_type})
    path = '/v3/marketing/paygiftactivity/activities'
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_paygift_merchant_delete(self, activity_id, merchant_id_list=[], delete_request_no=None):
    """删除活动发券商户号
    :param activity_id: 活动id，示例值:'10028001'
    :param delete_request_no: 请求业务单据号，商户创建批次凭据号（格式:商户id+日期+流水号），商户侧需保持唯一性，可包含英文字母，数字，｜，_，*，-等内容，不允许出现其他不合法符号。示例值:'100002322019090134234sfdf'
    :param merchant_id_list: 删除的发券商户号，从活动已有的发券商户号中移除的商户号列表，特殊规则:最小字符长度为8，最大为15，条目个数限制:[1，500]。示例值:["10000022"，"10000023"]
    """
    if activity_id:
        path = '/v3/marketing/paygiftactivity/activities/%s/merchants/delete' % activity_id
    else:
        raise Exception('activity_id is not assigned.')
    params = {}
    if merchant_id_list:
        params.update({'merchant_id_list': merchant_id_list})
    if delete_request_no:
        params.update({'delete_request_no': delete_request_no})
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_favor_stock_create(self,
                                 stock_name,
                                 belong_merchant,
                                 available_begin_time,
                                 available_end_time,
                                 stock_use_rule,
                                 coupon_use_rule,
                                 out_request_no,
                                 stock_type='NORMAL',
                                 no_cash=False,
                                 comment=None,
                                 pattern_info=None,
                                 ext_info=None):
    """创建代金券批次
    :param stock_name: 批次名称，示例值:'微信支付代金券批次'
    :param belong_merchant: 归属商户号。示例值:'98568865'
    :param available_begin_time: 可用时间-开始时间，格式为YYYY-MM-DDTHH:mm:ss.sss+TIMEZONE。示例值:'2015-05-20T13:29:35.120+08:00'
    :param available_end_time: 可用时间-结束时间，格式为YYYY-MM-DDTHH:mm:ss.sss+TIMEZONE。示例值:'2015-05-20T13:29:35.120+08:00'
    :param stock_use_rule: 发放规则。示例值:{'max_coupons':5, 'max_amount':100, 'max_coupons_per_user':1, 'natural_person_limit':False, 'prevent_api_abuse':True}
    :param coupon_use_rule: 核销规则。示例值:{'available_merchants':['9856000','9856111']}
    :param out_request_no: 商户单据号，可包含英文字母，数字，|，_，*，-等内容，不允许出现其他不合法符号，商户侧需保持商户单据号全局唯一。示例值:'89560002019101000121'
    :param stock_type: 批次类型，仅支持:'NORMAL':固定面额满减券批次。示例值:'NORMAL'
    :param no_cash: 营销经费，枚举值:True:免充值，False:预充值。示例值:False
    :param comment: 批次备注，仅制券商户可见，用于自定义信息。校验规则:批次备注最多60个UTF8字符数。示例值:'零售批次'
    :param pattern_info: 样式设置，示例值:{'description':'微信支付营销代金券'}
    :param ext_info: 扩展属性，json格式字符串，如无需要则不填写。示例值:"{'exinfo1':'1234','exinfo2':'3456'}"
    """
    params = {}
    if stock_name:
        params.update({'stock_name': stock_name})
    else:
        raise Exception('stock_name is not assigned.')
    if belong_merchant:
        params.update({'belong_merchant': belong_merchant})
    else:
        raise Exception('belong_merchant is not assigned.')
    if available_begin_time:
        params.update({'available_begin_time': available_begin_time})
    else:
        raise Exception('available_begin_time is not assigned.')
    if available_end_time:
        params.update({'available_end_time': available_end_time})
    else:
        raise Exception('available_end_time is not assigned.')
    if stock_use_rule:
        params.update({'stock_use_rule': stock_use_rule})
    else:
        raise Exception('stock_use_rule is not assigned.')
    if coupon_use_rule:
        params.update({'coupon_use_rule': coupon_use_rule})
    else:
        raise Exception('coupon_use_rule is not assigned.')
    if stock_type:
        params.update({'stock_type': stock_type})
    else:
        raise Exception('stock_type is not assigned.')
    if out_request_no:
        params.update({'out_request_no': out_request_no})
    else:
        raise Exception('out_request_no is not assigned.')
    if no_cash:
        params.update({'no_cash': no_cash})
    if comment:
        params.update({'comment': comment})
    if pattern_info:
        params.update({'pattern_info': pattern_info})
    if ext_info:
        params.update({'ext_info': ext_info})
    path = '/v3/marketing/favor/coupon-stocks'
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_favor_stock_start(self, stock_creator_mchid, stock_id):
    """激活代金券批次
    :param stock_creator_mchid: 创建批次的商户号，示例值:'8956000'
    :param stock_id: 批次号，示例值:'9856000'
    """
    params = {}
    if stock_creator_mchid:
        params.update({'stock_creator_mchid': stock_creator_mchid})
    else:
        raise Exception('stock_creator_mchid is not assigned.')
    if stock_id:
        path = '/v3/marketing/favor/stocks/%s/start' % stock_id
    else:
        raise Exception('stock_id is not assigned.')
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_favor_stock_send(self,
                               stock_id,
                               openid,
                               out_request_no,
                               stock_creator_mchid,
                               coupon_value=None,
                               coupon_minimum=None,
                               appid=None):
    """发放代金券批次
    :param stock_id: 批次号。示例值:'9856000'
    :param openid: 用户openid，示例值:'2323dfsdf342342'
    :param out_request_no: 商户单据号，示例值: '89560002019101000121'
    :param stock_creator_mchid: 创建批次的商户号，示例值:'8956000'
    :param coupon_value: 指定面额发券，面额。示例值:100
    :param coupon_minimum: 指定面额发券，券门槛。示例值:100
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    """
    params = {}
    if stock_id:
        params.update({'stock_id': stock_id})
    else:
        raise Exception('stock_id is not assigned.')
    if openid:
        path = '/v3/marketing/favor/users/%s/coupons' % openid
    else:
        raise Exception('openid is not assigned.')
    if out_request_no:
        params.update({'out_request_no': out_request_no})
    else:
        raise Exception('out_request_no is not assigned.')
    if stock_creator_mchid:
        params.update({'stock_creator_mchid': stock_creator_mchid})
    else:
        raise Exception('stock_creator_mchid is not assigned.')
    if coupon_value:
        params.update({'coupon_value': coupon_value})
    if coupon_minimum:
        params.update({'coupon_minimum': coupon_minimum})
    params.update({'appid': appid or self._appid})
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_favor_stock_pause(self, stock_creator_mchid, stock_id):
    """暂停代金券批次
    :param stock_creator_mchid: 创建批次的商户号，示例值:'8956000'
    :param stock_id: 批次号，示例值:'9856000'
    """
    params = {}
    if stock_creator_mchid:
        params.update({'stock_creator_mchid': stock_creator_mchid})
    else:
        raise Exception('stock_creator_mchid is not assigned.')
    if stock_id:
        path = '/v3/marketing/favor/stocks/%s/pause' % stock_id
    else:
        raise Exception('stock_id is not assigned.')
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_favor_stock_restart(self, stock_creator_mchid, stock_id):
    """重启代金券批次
    :param stock_creator_mchid: 创建批次的商户号，示例值:'8956000'
    :param stock_id: 批次号，示例值:'9856000'
    """
    params = {}
    if stock_creator_mchid:
        params.update({'stock_creator_mchid': stock_creator_mchid})
    else:
        raise Exception('stock_creator_mchid is not assigned.')
    if stock_id:
        path = '/v3/marketing/favor/stocks/%s/restart' % stock_id
    else:
        raise Exception('stock_id is not assigned.')
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_favor_stock_list(self,
                               stock_creator_mchid,
                               offset=0,
                               limit=10,
                               create_start_time=None,
                               create_end_time=None,
                               status=None):
    """条件查询批次列表
    :param stock_creator_mchid: 创建批次的商户号，示例值:'8956000'
    :param offset: 分页页码，页码从0开始，默认第0页。示例值:0
    :param limit: 分页大小，最大10。示例值:8
    :param create_start_time: 起始创建时间，格式为YYYY-MM-DDTHH:mm:ss.sss+TIMEZONE。示例值:'2015-05-20T13:29:35.120+08:00'
    :param create_end_time: 终止创建时间，格式为YYYY-MM-DDTHH:mm:ss.sss+TIMEZONE。示例值:'2015-05-20T13:29:35.120+08:00'
    :param status: 批次状态，枚举值:'unactivated':未激活，'audit':审核中，'running':运行中，'stoped':已停止，'paused':暂停发放。示例值:'paused'
    """
    if stock_creator_mchid:
        path = '/v3/marketing/favor/stocks?offset=%s&limit=%s&stock_creator_mchid=%s' % (offset, limit, stock_creator_mchid)
    else:
        raise Exception('stock_creator_mchid is not assigned.')
    if create_start_time:
        path = '%s&create_start_time=%s' % (path, create_start_time)
    if create_end_time:
        path = '%s&create_end_time=%s' % (path, create_end_time)
    if status:
        path = '%s&status=%s' % (path, status)
    return self._core.request(path)


def marketing_favor_stock_detail(self, stock_creator_mchid, stock_id):
    """查询批次详情
    :param stock_creator_mchid: 创建批次的商户号，示例值:'8956000'
    :param stock_id: 批次号，示例值:'9856000'
    """
    if stock_id:
        path = '/v3/marketing/favor/stocks/%s' % stock_id
    else:
        raise Exception('stock_id is not assigned.')
    if stock_creator_mchid:
        path = '%s?stock_creator_mchid=%s' % (path, stock_creator_mchid)
    else:
        raise Exception('stock_creator_mchid is not assigned.')
    return self._core.request(path)


def marketing_favor_coupon_detail(self, coupon_id, openid):
    """查询代金券详情
    :param coupon_id: 代金券id，示例值:'9856888'
    :param openid: 用户openid，示例值:'2323dfsdf342342'
    """
    if coupon_id and openid:
        path = '/v3/marketing/favor/users/%s/coupons/%s?appid=%s' % (openid, coupon_id, self._appid)
    else:
        raise Exception('coupon_id or openid is not assigned.')
    return self._core.request(path)


def marketing_favor_stock_merchant(self, stock_creator_mchid, stock_id, offset=0, limit=50):
    """查询代金券可用商户
    :param stock_creator_mchid: 创建批次的商户号，示例值:'8956000'
    :param stock_id: 批次号，示例值:'9856000'
    :param offset: 分页页码，最大1000。示例值: 10
    :param limit: 分页大小，最大50。示例值: 10
    """
    if stock_id:
        path = '/v3/marketing/favor/stocks/%s/merchants' % stock_id
    else:
        raise Exception('stock_id is not assigned.')
    if stock_creator_mchid:
        path = '%s?stock_creator_mchid=%s&offset=%s&limit=%s&' % (path, stock_creator_mchid, offset, limit)
    return self._core.request(path)


def marketing_favor_stock_item(self, stock_creator_mchid, stock_id, offset=0, limit=50):
    """查询代金券可用单品
    :param stock_creator_mchid: 创建批次的商户号，示例值:'8956000'
    :param stock_id: 批次号，示例值:'9856000'
    :param offset: 分页页码，最大500。示例值: 10
    :param limit: 分页大小，最大100。示例值: 10
    """
    if stock_id:
        path = '/v3/marketing/favor/stocks/%s/items' % stock_id
    else:
        raise Exception('stock_id is not assigned.')
    if stock_creator_mchid:
        path = '%s?stock_creator_mchid=%s&offset=%s&limit=%s&' % (path, stock_creator_mchid, offset, limit)
    return self._core.request(path)


def marketing_favor_user_coupon(self,
                                openid,
                                stock_id=None,
                                status=None,
                                creator_mchid=None,
                                sender_mchid=None,
                                available_mchid=None,
                                offset=0,
                                limit=20):
    """根据商户号查用户的券
    :param openid: 用户openid，示例值:'2323dfsdf342342'
    :param stock_id: 批次号，示例值:'9856000'
    :param status: 券状态，代金券状态:'SENDED':可用，'USED':已实扣，填写available_mchid参数则该字段不生效。示例值:'USED'
    :param creator_mchid: 创建批次的商户号.示例值:'9865002'
    :param sender_mchid: 批次发放商户号。示例值:'9865001'
    :param available_mchid: 可用商户号。示例值: '9865000'
    :param offset: 分页页码，默认0，填写available_mchid，该字段不生效。示例值:0
    :param limit: 分页大小，默认20，填写available_mchid，该字段不生效。示例值:20
    """
    if openid:
        path = '/v3/marketing/favor/users/%s/coupons?appid=%s&offset=%s&limit=%s' % (openid, self._appid, offset, limit)
    else:
        raise Exception('openid is not assigned.')
    if stock_id:
        path = '%s&stock_id=%s' % (path, stock_id)
    if status:
        path = '%s&status=%s' % (path, status)
    if creator_mchid:
        path = '%s&creator_mchid=%s' % (path, creator_mchid)
    elif sender_mchid:
        path = '%s&sender_mchid=%s' % (path, sender_mchid)
    elif available_mchid:
        path = '%s&available_mchid=%s' % (path, available_mchid)
    return self._core.request(path)


def marketing_favor_use_flow(self, stock_id):
    """下载批次核销明细
    :param stock_id: 批次号，微信为每个代金券批次分配的唯一id。示例值:'9865000'
    """
    if stock_id:
        path = '/v3/marketing/favor/stocks/%s/use-flow' % stock_id
    else:
        raise Exception('stock_id is not assigned.')
    return self._core.request(path)


def marketing_favor_refund_flow(self, stock_id):
    """下载批次退款明细
    :param stock_id: 批次号，微信为每个代金券批次分配的唯一id。示例值:'9865000'
    """
    if stock_id:
        path = '/v3/marketing/favor/stocks/%s/refund-flow' % stock_id
    else:
        raise Exception('stock_id is not assigned.')
    return self._core.request(path)


def marketing_favor_callback_update(self, notify_url=None, switch=True, mchid=None):
    """设置消息通知地址
    :param notify_url: 支付通知商户url地址。示例值:'https://pay.weixin.qq.com'
    :param switch: 回调开关，枚举值:True:开启推送，False:停止推送。示例值:True
    :param mchid: 微信支付商户号，可不填，默认传入初始化的mchid。示例值:'9856888'
    """
    params = {}
    params.update({'mchid': mchid or self._mchid})
    if not (notify_url or self._notify_url):
        raise Exception('notify_url is not assigned.')
    params.update({'notify_url': notify_url or self._notify_url})
    params.update({'switch': switch})
    path = '/v3/marketing/favor/callbacks'
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_busifavor_stock_create(self,
                                     stock_name,
                                     belong_merchant,
                                     goods_name,
                                     stock_type,
                                     coupon_use_rule,
                                     stock_send_rule,
                                     out_request_no,
                                     coupon_code_mode,
                                     comment=None,
                                     custom_entrance=None,
                                     display_pattern_info=None,
                                     notify_config=None,
                                     subsidy=False):
    """创建商家券
    :params stock_name: 商家券批次名称，字数上限为21个，一个中文汉字/英文字母/数字均占用一个字数。示例值:'8月1日活动券'
    :params belong_merchant: 批次归属商户号。注:普通直连模式，该参数为直连商户号。示例值:'10000022'
    :params goods_name: 适用商品范围，用来描述批次在哪些商品可用，会显示在微信卡包中。字数上限为15个。示例值:'xxx商品使用'
    :params stock_type: 批次类型，'NORMAL':固定面额满减券批次，'DISCOUNT':折扣券批次，'EXCHANGE':换购券批次。示例值:'NORMAL'
    :params coupon_use_rule: 核销规则。示例值:{'coupon_available_time':{}, 'fixed_normal_coupon':{}, 'use_method':'OFF_LINE', }
    :params stock_send_rule: 发放规则。示例值:{'max_coupons':100, 'max_coupons_per_user':5}
    :params out_request_no: 商户请求单号。示例值:'100002322019090134234sfdf'
    :params coupon_code_mode: 券code模式，枚举值:'WECHATPAY_MODE':系统分配券code。（固定22位纯数字），'MERCHANT_API':商户发放时接口指定券code，'MERCHANT_UPLOAD':商户上传自定义code，发券时系统随机选取上传的券code。示例值:'WECHATPAY_MODE'
    :params comment: 批次备注，仅配置商户可见，用于自定义信息。字数上限为20个。示例值:'活动使用'
    :params custom_entrance: 自定义入口。示例值:{'hall_id':'233455656'}
    :params display_pattern_info: 样式信息。示例值:{'description':'xxx门店可用'}
    :params notify_config: 事件通知配置。示例值:{'notify_appid':'wx23232232323'}
    :params subsidy=False: 是否允许营销补贴，该批次发放的券是否允许进行补差。示例值:False
    """
    params = {}
    if stock_name:
        params.update({'stock_name': stock_name})
    else:
        raise Exception('stock_name is not assigned.')
    if belong_merchant:
        params.update({'belong_merchant': belong_merchant})
    else:
        raise Exception('belong_merchant is not assigned.')
    if goods_name:
        params.update({'goods_name': goods_name})
    else:
        raise Exception('goods_name is not assigned.')
    if stock_type:
        params.update({'stock_type': stock_type})
    else:
        raise Exception('stock_type is not assigned.')
    if coupon_use_rule:
        params.update({'coupon_use_rule': coupon_use_rule})
    else:
        raise Exception('coupon_use_rule is not assigned.')
    if stock_send_rule:
        params.update({'stock_send_rule': stock_send_rule})
    else:
        raise Exception('stock_send_rule is not assigned.')
    if out_request_no:
        params.update({'out_request_no': out_request_no})
    else:
        raise Exception('out_request_no is not assigned.')
    if coupon_code_mode:
        params.update({'coupon_code_mode': coupon_code_mode})
    else:
        raise Exception('coupon_code_mode is not assigned.')
    if comment:
        params.update({'comment': comment})
    if custom_entrance:
        params.update({'custom_entrance': custom_entrance})
    if display_pattern_info:
        params.update({'display_pattern_info': display_pattern_info})
    if notify_config:
        params.update({'notify_config': notify_config})
    params.update({'subsidy': subsidy})
    path = '/v3/marketing/busifavor/stocks'
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_busifavor_stock_query(self, stock_id):
    """查询商家券详情
    :param stock_id: 批次号。示例值:1212
    """
    if stock_id:
        path = '/v3/marketing/busifavor/stocks/%s' % stock_id
    else:
        raise Exception('stock_id is not assigned.')
    return self._core.request(path)


def marketing_busifavor_coupon_use(self,
                                   coupon_code,
                                   use_time,
                                   use_request_no,
                                   stock_id=None,
                                   openid=None,
                                   appid=None):
    """核销用户券
    :param coupon_code: 券code，券的唯一标识。示例值:'sxxe34343434'
    :param use_time: 请求核销时间，格式为YYYY-MM-DDTHH:mm:ss+TIMEZONE。示例值:'2015-05-20T13:29:35+08:00'
    :param use_request_no: 核销请求单据号，每次核销请求的唯一标识，商户需保证唯一。示例值:'1002600620019090123143254435'
    :param stock_id: 批次号。示例值:1212
    :param openid: 用户标识。示例值:'xsd3434454567676'
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    """
    params = {}
    if coupon_code:
        params.update({'coupon_code': coupon_code})
    else:
        raise Exception('coupon_code is not assigned.')
    if use_time:
        params.update({'use_time': use_time})
    else:
        raise Exception('use_time is not assigned.')
    if use_request_no:
        params.update({'use_request_no': use_request_no})
    else:
        raise Exception('use_request_no is not assigned.')
    if stock_id:
        params.update({'stock_id': stock_id})
    if openid:
        params.update({'openid': openid})
    params.update({'appid': appid or self._appid})
    path = '/v3/marketing/busifavor/coupons/use'
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_busifavor_user_coupon(self,
                                    openid,
                                    stock_id=None,
                                    coupon_state=None,
                                    creator_merchant=None,
                                    belong_merchant=None,
                                    sender_merchant=None,
                                    offset=0,
                                    limit=20):
    """根据过滤条件查询用户券
    :param openid: 用户标识。示例值:'xsd3434454567676'
    :param stock_id: 批次号。示例值:1212
    :param coupon_state: 券状态，枚举值:'SENDED':可用，'USED':已核销，'EXPIRED':已过期，示例值:'SENDED'
    :param creator_merchant: 创建批次的商户号。示例值:'1000000001'
    :param belong_merchant: 批次归属商户号。示例值:'1000000002'
    :param sender_merchant: 批次发放商户号。示例值:'1000000003'
    :param offset: 分页页码。示例值:0
    :param limit: 分页大小。示例值:20
    """
    if openid:
        path = '/v3/marketing/busifavor/users/%s/coupons?appid=%s&offset=%s&limit=%s' % (openid, self._appid, offset, limit)
    else:
        raise Exception('openid is not assigned.')
    if stock_id:
        path = '%s&stock_id=%s' % (path, stock_id)
    if coupon_state:
        path = '%s&coupon_state=%s' % (path, coupon_state)
    if creator_merchant:
        path = '%s&creator_merchant=%s' % (path, creator_merchant)
    if belong_merchant:
        path = '%s&belong_merchant=%s' % (path, belong_merchant)
    if sender_merchant:
        path = '%s&sender_merchant=%s' % (path, sender_merchant)
    return self._core.request(path)


def marketing_busifavor_coupon_detail(self, coupon_code, openid):
    """查询用户单张券详情
    :param coupon_code: 券code，券的唯一标识。示例值:'sxxe34343434'
    :param openid: 用户标识。示例值:'xsd3434454567676'
    """
    if not (coupon_code and openid):
        raise Exception('coupon_code or openid is not assigned.')
    path = '/v3/marketing/busifavor/users/%s/coupons/%s/appids/%s' % (openid, coupon_code, self._appid)
    return self._core.request(path)


def marketing_busifavor_couponcode_upload(self,
                                          stock_id,
                                          upload_request_no,
                                          coupon_code_list=[]):
    """上传预存code
    :param stock_id: 批次号。示例值:1212
    :param upload_request_no: 请求业务单据号。商户上传code的凭据号，商户侧需保持唯一性。示例值:'100002322019090134234sfdf'
    :param coupon_code_list: 券code列表。示例值:['ABC9588200'，'ABC9588201']
    """
    params = {}
    if stock_id:
        path = '/v3/marketing/busifavor/stocks/%s/couponcodes' % stock_id
    else:
        raise Exception('stock_id is not assigned.')
    if upload_request_no:
        params.update({'upload_request_no': upload_request_no})
    else:
        raise Exception('upload_request_no is not assigned.')
    if coupon_code_list:
        params.update({'coupon_code_list': coupon_code_list})
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_busifavor_callback_update(self, mchid=None, notify_url=None):
    """设置商家券事件通知地址
    :param mchid: 商户号，可不填，默认传入初始化的mchid。示例值:'10000098'
    :param notify_url: 通知URL地址，用于接收商家券事件通知的url地址，不填默认使用初始化的notify_url。示例值:'https://pay.weixin.qq.com'
    """
    params = {}
    params.update({'mchid': mchid or self._mchid})
    if not (notify_url or self._notify_url):
        raise Exception('notify_url is not assigned.')
    params.update({'notify_url': notify_url or self._notify_url})
    path = '/v3/marketing/busifavor/callbacks'
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_busifavor_callback_query(self, mchid=None):
    """查询商家券事件通知地址
    :param mchid: 商户号，不填默认使用初始化的mchid。示例值:'10000098'
    """
    path = '/v3/marketing/busifavor/callbacks?mchid=%s' % (mchid or self._mchid)
    return self._core.request(path)


def marketing_busifavor_coupon_associate(self, stock_id, coupon_code, out_trade_no, out_request_no):
    """关联订单信息
    :param stock_id: 批次号。示例值:1212
    :param coupon_code: 券code，券的唯一标识。示例值:'sxxe34343434'
    :param out_trade_no: 关联的商户订单号，微信支付下单时的商户订单号，欲与该商家券关联的微信支付。示例值:'MCH_102233445'
    :param out_request_no: 商户请求单号，示例值:'1002600620019090123143254435'
    """
    params = {}
    if stock_id:
        params.update({'stock_id': stock_id})
    else:
        raise Exception('stock_id is not assigned.')
    if coupon_code:
        params.update({'coupon_code': coupon_code})
    else:
        raise Exception('coupon_code is not assigned.')
    if out_trade_no:
        params.update({'out_trade_no': out_trade_no})
    else:
        raise Exception('out_trade_no is not assigned.')
    if out_request_no:
        params.update({'out_request_no': out_request_no})
    else:
        raise Exception('out_request_no is not assigned.')
    path = '/v3/marketing/busifavor/coupons/associate'
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_busifavor_coupon_disassociate(self, stock_id, coupon_code, out_trade_no, out_request_no):
    """取消关联订单信息
    :param stock_id: 批次号。示例值:1212
    :param coupon_code: 券code，券的唯一标识。示例值:'sxxe34343434'
    :param out_trade_no: 关联的商户订单号，微信支付下单时的商户订单号，欲与该商家券关联的微信支付。示例值:'MCH_102233445'
    :param out_request_no: 商户请求单号，示例值:'1002600620019090123143254435'
    """
    params = {}
    if stock_id:
        params.update({'stock_id': stock_id})
    else:
        raise Exception('stock_id is not assigned.')
    if coupon_code:
        params.update({'coupon_code': coupon_code})
    else:
        raise Exception('coupon_code is not assigned.')
    if out_trade_no:
        params.update({'out_trade_no': out_trade_no})
    else:
        raise Exception('out_trade_no is not assigned.')
    if out_request_no:
        params.update({'out_request_no': out_request_no})
    else:
        raise Exception('out_request_no is not assigned.')
    path = '/v3/marketing/busifavor/coupons/disassociate'
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_busifavor_stock_budget(self,
                                     stock_id,
                                     modify_budget_request_no,
                                     target_max_coupons=None,
                                     target_max_coupons_by_day=None,
                                     current_max_coupons=None,
                                     current_max_coupons_by_day=None):
    """修改批次预算
    :param stock_id: 批次号。示例值:1212
    :param modify_budget_request_no: 修改预算请求单据号，示例值:'1002600620019090123143254436'
    :param target_max_coupons: 目标批次最大发放个数。示例值:3000
    :param target_max_coupons_by_day: 目标单天发放上限个数。示例值:500
    :param current_max_coupons: 当前批次最大发放个数。示例值:500
    :param current_max_coupons_by_day: 当前单天发放上限个数。示例值:300
    """
    params = {}
    if stock_id:
        path = '/v3/marketing/busifavor/stocks/%s/budget' % stock_id
    else:
        raise Exception('stock_id is not assigned.')
    if modify_budget_request_no:
        params.update({'modify_budget_request_no': modify_budget_request_no})
    else:
        raise Exception('modify_budget_request_no is not assigned.')
    if target_max_coupons:
        params.update({'target_max_coupons': target_max_coupons})
    elif target_max_coupons_by_day:
        params.update({'target_max_coupons_by_day': target_max_coupons_by_day})
    else:
        raise Exception('target_max_coupons or target_max_coupons_by_day is not assigned.')
    if current_max_coupons:
        params.update({'current_max_coupons': current_max_coupons})
    if current_max_coupons_by_day:
        params.update({'current_max_coupons_by_day': current_max_coupons_by_day})
    return self._core.request(path, method=RequestType.PATCH, data=params)


def marketing_busifavor_stock_modify(self,
                                     stock_id,
                                     out_request_no,
                                     custom_entrance=None,
                                     comment=None,
                                     goods_name=None,
                                     display_pattern_info=None,
                                     coupon_use_rule=None,
                                     stock_send_rule=None,
                                     notify_config=None):
    """修改商家券基本信息
    :param stock_id: 批次号。示例值:1212
    :param out_request_no: 商户请求单号，示例值:'1002600620019090123143254435'
    :param custom_entrance: 自定义入口。示例值:{'hall_id':'234567'}
    :param comment: 批次备注，字数上限为20个。示例值:'活动使用'
    :param goods_name: 适用商品范围。示例值:'xxx商品使用'
    :param display_pattern_info: 样式信息。示例值:{'description':'xxx门店可用'}
    :param coupon_use_rule: 核销规则。示例值:{'use_method':'OFF_LINE'}
    :param stock_send_rule: 发放规则。示例值:{'prevent_api_abuse':False}
    :param notify_config: 事件通知配置。示例值:{'notify_appid':'wx23232232323'}
    """
    if stock_id:
        path = '/v3/marketing/busifavor/stocks/%s' % stock_id
    else:
        raise Exception('stock_id is not assigned.')
    params = {}
    if out_request_no:
        params.update({'out_request_no': out_request_no})
    else:
        raise Exception('out_request_no is not assigned.')
    if custom_entrance:
        params.update({'custom_entrance': custom_entrance})
    if comment:
        params.update({'comment': comment})
    if goods_name:
        params.update({'goods_name': goods_name})
    if display_pattern_info:
        params.update({'display_pattern_info': display_pattern_info})
    if coupon_use_rule:
        params.update({'coupon_use_rule': coupon_use_rule})
    if stock_send_rule:
        params.update({'stock_send_rule': stock_send_rule})
    if notify_config:
        params.update({'notify_config': notify_config})
    return self._core.request(path, method=RequestType.PATCH, data=params)


def marketing_busifavor_coupon_return(self, coupon_code, stock_id, return_request_no):
    """申请退券
    :param coupon_code: 券code，券的唯一标识。示例值:'sxxe34343434'
    :param stock_id: 批次号。示例值:1212
    :param return_request_no: 退券请求单据号。示例值:'1002600620019090123143254436'
    """
    params = {}
    if coupon_code:
        params.update({'coupon_code': coupon_code})
    else:
        raise Exception('coupon_code is not assigned.')
    if stock_id:
        params.update({'stock_id': stock_id})
    else:
        raise Exception('stock_id is not assigned.')
    if return_request_no:
        params.update({'return_request_no': return_request_no})
    else:
        raise Exception('return_request_no is not assigned.')
    path = '/v3/marketing/busifavor/coupons/return'
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_busifavor_coupon_deactivate(self, coupon_code, stock_id, deactivate_request_no, deactivate_reason=None):
    """使券失效
    :param coupon_code: 券code，券的唯一标识。示例值:'sxxe34343434'
    :param stock_id: 批次号。示例值:1212
    :param deactivate_request_no: 失效请求单据号。示例值:'1002600620019090123143254436'
    :param deactivate_reason: 失效原因。示例值:'此券使用时间设置错误'
    """
    params = {}
    if coupon_code:
        params.update({'coupon_code': coupon_code})
    else:
        raise Exception('coupon_code is not assigned.')
    if stock_id:
        params.update({'stock_id': stock_id})
    else:
        raise Exception('stock_id is not assigned.')
    if deactivate_request_no:
        params.update({'deactivate_request_no': deactivate_request_no})
    else:
        raise Exception('deactivate_request_no is not assigned.')
    if deactivate_reason:
        params.update({'deactivate_reason': deactivate_reason})
    path = '/v3/marketing/busifavor/coupons/deactivate'
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_busifavor_subsidy_pay(self,
                                    stock_id,
                                    coupon_code,
                                    transaction_id,
                                    payer_merchant,
                                    payee_merchant,
                                    amount,
                                    description,
                                    out_subsidy_no):
    """营销补差付款
    :param stock_id: 批次号。示例值:1212
    :param coupon_code: 券code，券的唯一标识。示例值:'sxxe34343434'
    :param transaction_id: 微信支付订单号。示例值:'4200000913202101152566792388'
    :param payer_merchant: 营销补差扣款商户号。示例值:'1900000001'
    :param payee_merchant: 营销补差入账商户号。示例值:'1900000002'
    :param amount: 补差付款金额。示例值:100
    :param description: 补差付款描述。示例值:'20210115DESCRIPTION'
    :param out_subsidy_no: 业务请求唯一单号。示例值:'subsidy-abcd-12345678'
    """
    params = {}
    if coupon_code:
        params.update({'coupon_code': coupon_code})
    else:
        raise Exception('coupon_code is not assigned.')
    if stock_id:
        params.update({'stock_id': stock_id})
    else:
        raise Exception('stock_id is not assigned.')
    if transaction_id:
        params.update({'transaction_id': transaction_id})
    else:
        raise Exception('transaction_id is not assigned.')
    if payer_merchant:
        params.update({'payer_merchant': payer_merchant})
    else:
        raise Exception('payer_merchant is not assigned.')
    if payee_merchant:
        params.update({'payee_merchant': payee_merchant})
    else:
        raise Exception('payee_merchant is not assigned.')
    if amount:
        params.update({'amount': amount})
    else:
        raise Exception('amount is not assigned.')
    if description:
        params.update({'description': description})
    else:
        raise Exception('description is not assigned.')
    if out_subsidy_no:
        params.update({'out_subsidy_no': out_subsidy_no})
    else:
        raise Exception('out_subsidy_no is not assigned.')
    path = '/v3/marketing/busifavor/subsidy/pay-receipts'
    return self._core.request(path, method=RequestType.POST, data=params)


def marketing_busifavor_subsidy_query(self, subsidy_receipt_id):
    """查询营销补差付款单详情
    :param subsidy_receipt_id: 补差付款单号。示例值:'1120200119165100000000000001'
    """
    if subsidy_receipt_id:
        path = '/v3/marketing/busifavor/subsidy/pay-receipts/%s' % subsidy_receipt_id
    else:
        raise Exception('subsidy_receipt_id is not assigned.')
    return self._core.request(path)


def industry_coupon_token(self, open_id, coupon_list=[]):
    """出行券切卡组件预下单
    https://pay.weixin.qq.com/wiki/doc/apiv3_partner/apis/chapter9_9_1.shtml
    :param open_id: 用户在商户AppID下的唯一标识，该用户为后续拉起切卡组件的用户。示例值：'obLatjrR8kUDlj4-nofQsPAJAAFI'
    :param coupon_list: 用户最近领取的出行券列表。示例值：[{"coupon_id": "11004999626", "stock_id": 16474341}]
    """
    params = {}
    if open_id:
        params.update({'open_id': open_id})
    else:
        raise Exception('open_id is not assigned.')
    if coupon_list:
        params.update({'coupon_list': coupon_list})
    else:
        raise Exception('coupon_list is not assigned.')
    path = '/v3/industry-coupon/tokens'
    return self._core.request(path, method=RequestType.POST, data=params)


def bank_package_file(self, package_id, bank_type, filepath):
    """导入定向用户协议号
    https://pay.weixin.qq.com/wiki/doc/apiv3_partner/apis/chapter9_8_1.shtml
    :package_id: 号码包唯一标识符。可在微信支付商户平台创建号码包后获得。示例值：'8473295'
    :filepath: 电子发票文件路径，只支持txt和csv两种格式，示例值：'./active_user.csv'
    """
    if not (filepath and os.path.exists(filepath) and os.path.isfile(filepath)):
        raise Exception('filepath is not assigned or not exists')
    with open(filepath, mode='rb') as f:
        content = f.read()
    filename = os.path.basename(filepath)
    filetype = os.path.splitext(filename)[-1][1:].upper()
    mimes = {
        'TXT': ' text/plain',
        'CSV': 'text/csv'
    }
    if filetype not in mimes:
        raise Exception('wechatpayv3 does not support this file type.')
    if not package_id or bank_type:
        raise Exception('package_id or bank_type is not assigned.')
    params = {}
    params.update({'meta': '{"bank_type":"%s", "filename":"%s", "sha256":"%s"}' % (bank_type, filename, sha256(content))})
    files = [('file', (filename, content, mimes[filetype]))]
    path = '/v3/marketing/bank/packages/%s/tasks' % package_id
    return self._core.request(path, method=RequestType.POST, data=params, sign_data=params.get('meta'), files=files)



# -*- coding: utf-8 -*-

from enum import Enum, unique


@unique
class RequestType(Enum):
    GET = 'GET'
    POST = 'POST'
    PATCH = 'PATCH'
    PUT = 'PUT'
    DELETE = 'DELETE'


class WeChatPayType(Enum):
    JSAPI = 0
    APP = 1
    H5 = 2
    NATIVE = 3
    MINIPROG = 4


class SignType(Enum):
    RSA_SHA256 = 0
    HMAC_SHA256 = 1
    MD5 = 2



# -*- coding: utf-8 -*-


def capital_search_bank_number(self, account_number):
    """获取对私银行卡号开户银行
    :param account_number: 银行卡号，示例值：'1234567890123'
    """
    from urllib.parse import urlencode
    params = {}
    params.update({'account_number': self._core.encrypt(account_number)})
    path = '/v3/capital/capitallhh/banks/search-banks-by-bank-account?%s' % urlencode(params)
    return self._core.request(path, cipher_data=True)


def capital_personal_banks(self, offset=0, limit=200):
    """查询支持个人业务的银行列表
    :param offset: 本次查询偏移量，示例值：0
    :param offset: 本次请求最大查询条数，示例值：200
    """
    path = '/v3/capital/capitallhh/banks/personal-banking?offset=%s&limit=%s' % (offset, limit)
    return self._core.request(path)


def capital_corporate_banks(self, offset=0, limit=200):
    """查询支持对公业务的银行列表
    :param offset: 本次查询偏移量，示例值：0
    :param offset: 本次请求最大查询条数，示例值：200
    """
    path = '/v3/capital/capitallhh/banks/corporate-banking?offset=%s&limit=%s' % (offset, limit)
    return self._core.request(path)


def capital_provinces(self):
    """查询省份列表
    """
    path = '/v3/capital/capitallhh/areas/provinces'
    return self._core.request(path)


def capital_cities(self, province_code):
    """查询城市列表
    :param province_code: 省份编码，唯一标识一个省份。示例值：10
    """
    path = '/v3/capital/capitallhh/areas/provinces/%s/cities' % province_code
    return self._core.request(path)


def capital_branches(self, bank_alias_code, city_code, offset=0, limit=100):
    """查询支行列表
    :param bank_alias_code: 银行别名的编码，查询支行接口仅支持需要填写支行的银行别名编码。示例值：1000006247
    :param city_code: 城市编码，唯一标识一座城市，用于结合银行别名编码查询支行列表。示例值：536
    :param offset: 本次查询偏移量，示例值：0
    :param offset: 本次请求最大查询条数，示例值：100
    """
    if bank_alias_code and city_code:
        path = '/v3/capital/capitallhh/banks/%s/branches?city_code=%s&offset=%s&limit=%s' % (bank_alias_code, city_code, offset, limit)
    else:
        raise Exception('bank_alias_code or city_code is not assigned.')
    return self._core.request(path)



# -*- coding: utf-8 -*-

from .type import RequestType


def transfer_batch(self, out_batch_no, batch_name, batch_remark, total_amount, total_num, transfer_detail_list=[], appid=None, transfer_scene_id=None):
    """发起商家转账
    :param out_batch_no: 商户系统内部的商家批次单号，要求此参数只能由数字、大小写字母组成，在商户系统内部唯一，示例值：'plfk2020042013'
    :param batch_name: 该笔批量转账的名称，示例值：'2019年1月深圳分部报销单'
    :param batch_remark: 转账说明，UTF8编码，最多允许32个字符，示例值：'2019年1月深圳分部报销单'
    :param total_amount: 转账总金额，单位为分，必须与批次内所有明细转账金额之和保持一致，否则无法发起转账操作，示例值：'4000000'
    :param total_num: 转账总笔数，必须与批次内所有明细之和保持一致，否则无法发起转账操作，示例值：200
    :param transfer_detail_list: 发起批量转账的明细列表，最多三千笔，示例值：[{"out_detail_no": "x23zy545Bd5436", "transfer_amount": 200000, "transfer_remark": "2020年4月报销", "openid": "o-MYE42l80oelYMDE34nYD456Xoy", "user_name": "张三"}]
    :param appid: 应用ID，可不填，默认传入初始化时的appid，示例值:'wx1234567890abcdef'
    :param transfer_scene_id: 转账场景ID，示例值:'1001'
    """
    params = {}
    if out_batch_no:
        params.update({'out_batch_no': out_batch_no})
    else:
        raise Exception('out_batch_no is not assigned')
    if batch_name:
        params.update({'batch_name': batch_name})
    else:
        raise Exception('batch_name is not assigned')
    if batch_remark:
        params.update({'batch_remark': batch_remark})
    else:
        raise Exception('batch_remark is not assigned')
    if total_amount:
        params.update({'total_amount': total_amount})
    else:
        raise Exception('total_amount is not assigned')
    if total_num:
        params.update({'total_num': total_num})
    else:
        raise Exception('total_num is not assigned')
    if transfer_detail_list:
        params.update({'transfer_detail_list': transfer_detail_list})
    else:
        raise Exception('transfer_detail_list is not assigned')
    cipher_data = False
    for transfer_detail in params.get('transfer_detail_list'):
        if transfer_detail.get('user_name'):
            transfer_detail['user_name'] = self._core.encrypt(transfer_detail.get('user_name'))
            cipher_data = True
    params.update({'appid': appid or self._appid})
    if transfer_scene_id:
        params.update({'transfer_scene_id': transfer_scene_id})
    path = '/v3/transfer/batches'
    return self._core.request(path, method=RequestType.POST, data=params, cipher_data=cipher_data)


def transfer_query_batchid(self, batch_id, need_query_detail=False, offset=0, limit=20, detail_status='ALL'):
    """微信批次单号查询批次单
    :param batch_id: 微信批次单号，微信商家转账系统返回的唯一标识，示例值：1030000071100999991182020050700019480001
    :param need_query_detail: 是否查询转账明细单，枚举值：true：是；false：否，默认否。
    :param offset: 请求资源起始位置，默认值为0
    :param limit: 最大资源条数，默认值为20
    :param detail_status: 明细状态， ALL:全部。需要同时查询转账成功和转账失败的明细单；SUCCESS:转账成功。只查询转账成功的明细单；FAIL:转账失败。
    """
    if batch_id:
        path = '/v3/transfer/batches/batch-id/%s' % batch_id
    else:
        raise Exception('batch_id is not assigned')
    if need_query_detail:
        path += '?need_query_detail=true'
    else:
        path += '?need_query_detail=false'
    path += '&offset=%s' % offset
    path += '&limit=%s' % limit
    path += '&detail_status=%s' % detail_status
    return self._core.request(path)


def transfer_query_detail_id(self, batch_id, detail_id):
    """微信明细单号查询明细单
    :param batch_id: 微信批次单号，微信商家转账系统返回的唯一标识，示例值：1030000071100999991182020050700019480001
    :param detail_id: 微信明细单号，微信支付系统内部区分转账批次单下不同转账明细单的唯一标识，示例值：1040000071100999991182020050700019500100
    """
    if batch_id and detail_id:
        path = '/v3/transfer/batches/batch-id/%s/details/detail-id/%s' % (batch_id, detail_id)
    else:
        raise Exception('batch_id or detail_id is not assigned')
    return self._core.request(path)


def transfer_query_out_batch_no(self, out_batch_no, need_query_detail=False, offset=0, limit=20, detail_status='ALL'):
    """商家批次单号查询批次单
    :param out_batch_no: 商家批次单号，示例值：plfk2020042013
    :param need_query_detail: 是否查询转账明细单，枚举值：true：是；false：否，默认否。
    :param offset: 请求资源起始位置，默认值为0
    :param limit: 最大资源条数，默认值为20
    :param detail_status: 明细状态， ALL:全部。需要同时查询转账成功和转账失败的明细单；SUCCESS:转账成功。只查询转账成功的明细单；FAIL:转账失败。
    """
    if out_batch_no:
        path = '/v3/transfer/batches/out-batch-no/%s' % out_batch_no
    else:
        raise Exception('batch_id is not assigned')
    if need_query_detail:
        path += '?need_query_detail=true'
    else:
        path += '?need_query_detail=false'
    path += '&offset=%s' % offset
    path += '&limit=%s' % limit
    path += '&detail_status=%s' % detail_status
    return self._core.request(path)


def transfer_query_out_detail_no(self, out_detail_no, out_batch_no):
    """商家明细单号查询明细单
    :param out_detail_no: 商家明细单号，示例值：x23zy545Bd5436
    :param out_batch_no: 商家批次单号，示例值：plfk2020042013
    """
    if out_detail_no and out_batch_no:
        path = '/v3/transfer/batches/out-batch-no/%s/details/out-detail-no/%s' % (out_batch_no, out_detail_no)
    else:
        raise Exception('out_detail_no or out_batch_no is not assigned')
    return self._core.request(path)


def transfer_bill_receipt(self, out_batch_no):
    """转账电子回单申请受理
    :param out_batch_no: 商家批次单号，示例值：plfk2020042013
    """
    params = {}
    if out_batch_no:
        params.update({'out_batch_no': out_batch_no})
    else:
        raise Exception('out_batch_no is assigned')
    path = '/v3/transfer/bill-receipt'
    return self._core.request(path, method=RequestType.POST, params=params)


def transfer_query_bill_receipt(self, out_batch_no):
    """查询转账电子回单
    :param out_batch_no: 商家批次单号，示例值：plfk2020042013
    """
    if out_batch_no:
        path = '/v3/transfer/bill-receipt/%s' % out_batch_no
    else:
        raise Exception('out_batch_no is not assigned')
    return self._core.request(path)


def transfer_detail_receipt(self, accept_type, out_detail_no, out_batch_no=None,):
    """转账明细电子回单受理
    :param accept_type: 受理类型
    :param out_detail_no: 商家明细单号，示例值：x23zy545Bd5436
    :param out_batch_no: 商家批次单号，示例值：plfk2020042013
    """
    params = {}
    if accept_type:
        params.update({'accept_type': accept_type})
    else:
        raise Exception('accept_type is not assigned')
    if out_detail_no:
        params.update({'out_detail_no': out_detail_no})
    else:
        raise Exception('out_detail_no is not assigned')
    if out_batch_no:
        params.update({'out_batch_no': out_batch_no})
    path = '/v3/transfer-detail/electronic-receipts'
    return self._core.request(path, method=RequestType.POST, params=params)


def transfer_query_receipt(self, accept_type, out_detail_no, out_batch_no=None):
    """查询转账明细电子回单受理结果
    :param accept_type: 受理类型
    :param out_detail_no: 商家明细单号，示例值：x23zy545Bd5436
    :param out_batch_no: 商家批次单号，示例值：plfk2020042013
    """
    if accept_type:
        path = '/v3/transfer-detail/electronic-receipts?accept_type=%s' % accept_type
    else:
        raise Exception('accept_type is not assigned')
    if out_detail_no:
        path += '&out_batch_no=%s' % out_detail_no
    else:
        raise Exception('out_detail_no is not assigned')
    if out_batch_no:
        path += '&out_batch_no=%s' % out_batch_no
    return self._core_request(path)



# -*- coding: utf-8 -*-

from .type import RequestType


def guides_register(self, corpid, store_id, userid, name, mobile, qr_code, avatar, group_qrcode=None, sub_mchid=None):
    """服务人员注册
    :param corpid: 企业ID, 示例值:'1234567890'
    :param store_id: 门店ID, 示例值:12345678
    :param userid: 企业微信的员工ID, 示例值:'robert'
    :param name: 企业微信的员工姓名, 示例值:'robert'
    :param mobile: 手机号码, 示例值:'13900000000'
    :param qr_code: 员工个人二维码, 示例值:'https://open.work.weixin.qq.com/wwopen/userQRCode?vcode=xxx'
    :param avatar: 头像URL, 示例值:'http://wx.qlogo.cn/mmopen/ajNVdqHZLLA3WJ6DSZUfiakYe37PKnQhBIeOQBO4czqrnZDS79FH5Wm5m4X69TBicnHFlhiafvDwklOpZeXYQQ2icg/0'
    :param group_qrcode: 群二维码URL, 示例值:'http://p.qpic.cn/wwhead/nMl9ssowtibVGyrmvBiaibzDtp/0'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    params = {}
    if corpid:
        params.update({'corpid': corpid})
    else:
        raise Exception('corpid is not assigned.')
    if store_id:
        params.update({'store_id': store_id})
    else:
        raise Exception('store_id is not assigned.')
    if userid:
        params.update({'userid': userid})
    else:
        raise Exception('userid is not assigned.')
    if name:
        params.update({'name': self._core.encrypt(name)})
    else:
        raise Exception('name is not assigned')
    if mobile:
        params.update({'mobile': self._core.encrypt(mobile)})
    else:
        raise Exception('mobile is not assigned.')
    if qr_code:
        params.update({'qr_code': qr_code})
    else:
        raise Exception('qr_code is not assigned.')
    if avatar:
        params.update({'avatar': avatar})
    else:
        raise Exception('avatar is not assigned.')
    if group_qrcode:
        params.update({'group_qrcode': group_qrcode})
    if self._partner_mode and sub_mchid:
        params.update({'sub_mchid': sub_mchid})
    path = '/v3/smartguide/guides'
    return self._core.request(path, method=RequestType.POST, data=params, cipher_data=True)


def guides_assign(self, guide_id, out_trade_no, sub_mchid=None):
    """服务人员分配
    :param guide_id: 服务人员ID，示例值:'LLA3WJ6DSZUfiaZDS79FH5Wm5m4X69TBic'
    :param out_trade_no: 商户订单号, 示例值:'20150806125346'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    params = {}
    if out_trade_no:
        params.update({'out_trade_no': out_trade_no})
    else:
        raise Exception('out_trade_no is not assigned.')
    if self._partner_mode and sub_mchid:
        params.update({'sub_mchid': sub_mchid})
    if guide_id:
        path = '/v3/smartguide/guides/%s/assign' % guide_id
    else:
        raise Exception('guide_id is not assigned.')
    return self._core.request(path, method=RequestType.POST, data=params)


def guides_query(self, store_id, userid=None, mobile=None, work_id=None, limit=None, offset=0, sub_mchid=None):
    """服务人员查询
    :params store_id: 门店ID, 示例值:1234
    :params userid: 企业微信的员工ID, 示例值:'robert'
    :params mobile: 手机号码, 示例值:'13900000000'
    :params work_id: 工号, 示例值:'robert'
    :params limit: 最大资源条数, 示例值:5
    :params offset: 请求资源起始位置, 示例值:0
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    if not store_id:
        raise Exception('store_id is not assigned.')
    path = '/v3/smartguide/guides?store_id=%s' % store_id
    if userid:
        path = '%s&userid=%s' % (path, userid)
    cipher_data = False
    if mobile:
        path = '%s&mobile=%s' % (path, self._core.encrypt(mobile))
        cipher_data = True
    if work_id:
        path = '%s&work_id=%s' % (path, work_id)
    if limit:
        path = '%s&limit=%s' % (path, limit)
    if offset:
        path = '%s&offset=%s' % (path, offset)
    if self._partner_mode and sub_mchid:
        path = '%s&sub_mchid=%s' % (path, sub_mchid)
    return self._core.request(path, cipher_data=cipher_data)


def guides_update(self, guide_id, name=None, mobile=None, qr_code=None, avatar=None, group_qrcode=None, sub_mchid=None):
    """服务人员信息更新
    :params guide_id: 服务人员ID, 示例值:'LLA3WJ6DSZUfiaZDS79FH5Wm5m4X69TBic'
    :params name: 服务人员姓名, 示例值:'robert'
    :params mobile: 服务人员手机号码, 示例值:'13900000000'
    :params qr_code: 服务人员二维码URL, 示例值:'https://open.work.weixin.qq.com/wwopen/userQRCode?vcode=xxx'
    :params avatar: 服务人员头像URL, 示例值:'http://wx.qlogo.cn/mmopen/ajNVdqHZLLA3WJ6DSZUfiakYe37PKnQhBIeOQBO4czqrnZDS79FH5Wm5m4X69TBicnHFlhiafvDwklOpZeXYQQ2icg/0'
    :params group_qrcode: 群二维码URL, 示例值:'http://p.qpic.cn/wwhead/nMl9ssowtibVGyrmvBiaibzDtp/0'
    :param sub_mchid: (服务商模式)子商户的商户号，由微信支付生成并下发。示例值:'1900000109'
    """
    params = {}
    if not guide_id:
        raise Exception('guide_id is not assigned.')
    path = '/v3/smartguide/guides/%s' % guide_id
    cipher_data = False
    if name:
        params.update({'name': self._core.encrypt(name)})
        cipher_data = True
    if mobile:
        params.update({'mobile': self._core.encrypt(mobile)})
        cipher_data = True
    if qr_code:
        params.update({'qr_code': qr_code})
    if avatar:
        params.update({'avatar': avatar})
    if group_qrcode:
        params.update({'group_qrcode': group_qrcode})
    if self._partner_mode and sub_mchid:
        params.update({'sub_mchid': sub_mchid})
    return self._core.request(path, method=RequestType.PATCH, data=params, cipher_data=cipher_data)



# -*- coding: utf-8 -*-

from .type import RequestType


def merchantrisk_callback_create(self, notify_url=None):
    """创建商户违规通知回调地址
    :param notify_url: 通知地址，示例值:'https://www.weixin.qq.com/wxpay/pay.php'
    """
    params = {}
    if notify_url:
        params.update({'notify_url': notify_url})
    path = '/v3/merchant-risk-manage/violation-notifications'
    return self._core.request(path, method=RequestType.POST, data=params)


def merchantrisk_callback_query(self):
    """查询商户违规通知回调地址
    """
    path = '/v3/merchant-risk-manage/violation-notifications'
    return self._core.request(path)


def merchantrisk_callback_update(self, notify_url=None):
    """修改商户违规通知回调地址
    :param notify_url: 通知地址，示例值:'https://www.weixin.qq.com/wxpay/pay.php'
    """
    params = {}
    if notify_url:
        params.update({'notify_url': notify_url})
    path = '/v3/merchant-risk-manage/violation-notifications'
    return self._core.request(path, method=RequestType.PUT, data=params)


def merchantrisk_callback_delete(self):
    """查询商户违规通知回调地址
    """
    path = '/v3/merchant-risk-manage/violation-notifications'
    return self._core.request(path, method=RequestType.DELETE)



# -*- coding: utf-8 -*-

import os.path

from .type import RequestType
from .utils import sha256


def _media_upload(self, filepath, filename, path):
    if not (filepath and os.path.exists(filepath) and os.path.isfile(filepath) and path):
        raise Exception('filepath is not assigned or not exists')
    with open(filepath, mode='rb') as f:
        content = f.read()
    if not filename:
        filename = os.path.basename(filepath)
    params = {}
    params.update({'meta': '{"filename":"%s","sha256":"%s"}' % (filename, sha256(content))})
    mimes = {
        '.bmp': 'image/bmp',
        '.jpg': 'image/jpeg',
        '.jpeg': 'image/jpeg',
        '.png': 'image/png',
        '.avi': 'video/x-msvideo',
        '.wmv': 'video/x-ms-wmv',
        '.mpeg': 'video/mpeg',
        '.mp4': 'video/mp4',
        '.mov': 'video/quicktime',
        '.mkv': 'video/x-matroska',
        '.flv': 'video/x-flv',
        '.f4v': 'video/x-f4v',
        '.m4v': 'video/x-m4v',
        '.rmvb': 'application/vnd.rn-realmedia-vbr'
    }
    media_type = os.path.splitext(filename)[-1]
    if media_type not in mimes:
        raise Exception('wechatpayv3 does not support this media type.')
    files = [('file', (filename, content, mimes[media_type]))]
    return self._core.request(path, method=RequestType.POST, data=params, sign_data=params.get('meta'), files=files)


def image_upload(self, filepath, filename=None):
    """图片上传
    :param filepath: 图片文件路径
    :param filename: 文件名称，未指定则从filepath参数中截取
    """
    return _media_upload(self, filepath, filename, path='/v3/merchant/media/upload')


def video_upload(self, filepath, filename=None):
    """视频上传
    :param filepath: 视频文件路径
    :param filename: 文件名称，未指定则从filepath参数中截取
    """
    return _media_upload(self, filepath, filename, path='/v3/merchant/media/video_upload')
